/*
 * AlarmSettingsWindow.java
 *
 * Created on December 25, 2004, 2:24 PM
 */

/**
 *
 * @author  Andrew Flowers, K0SM
 */



import java.io.*;
import java.util.prefs.*;

/**
 * This class is a frame that contains the Alarm Settings for RainScatter.  The
 * user can select an alarm type (popups or no alarm) and conditions that, when
 * met, will cause the alarm to activate.  This class is a window to change
 * those settings.  Those settings are saved to disk when the user presses
 * "OK".  RainScatter can query for the state of the alarm through the public
 * methods in this class.
 */
public class AlarmSettingsWindow extends javax.swing.JFrame implements java.awt.event.ActionListener {
    
    private Preferences root = Preferences.userRoot();
    final Preferences node = root.node("gui/rainscatter");
    
    public final int ALARM_ON = 1;
    public final int ALARM_OFF = 0;
    public final int ANY_VISIBILE_STORMS = 1;
    public final int DX_OR_STATIONS = 2;
    public final int STATIONS_ONLY = 3;
    public final int DX_ONLY = 4;
    
    private int alarmType;
    private int alarmCondition;
    
    //used to call back to main program (for updating GUI)
    private final RainScatter MAIN_PROGRAM = (RainScatter)RainScatter.getFrames()[0];
    
    /** Creates new form AlarmSettingsWindow */
    public AlarmSettingsWindow() {
        initComponents();
        readAlarmSettings();
        alarm1RadioButton.addActionListener(this);
        alarm2RadioButton.addActionListener(this);
        alarm3RadioButton.addActionListener(this);
        alarm4RadioButton.addActionListener(this);
        
        jCheckBox1.addActionListener(this);
        MAIN_PROGRAM.updateAlarmLabel();  //update GUI in main program
                                         // to reflect current alarm state
        setSize(390,390);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        alarmConditionsButtonGroup = new javax.swing.ButtonGroup();
        alarmTypeButtonGroup = new javax.swing.ButtonGroup();
        jPanel1 = new javax.swing.JPanel();
        alarm1RadioButton = new javax.swing.JRadioButton();
        alarm2RadioButton = new javax.swing.JRadioButton();
        alarm3RadioButton = new javax.swing.JRadioButton();
        alarm4RadioButton = new javax.swing.JRadioButton();
        jButton1 = new javax.swing.JButton();
        jCheckBox1 = new javax.swing.JCheckBox();

        getContentPane().setLayout(null);

        setTitle("Alarm Settings");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        jPanel1.setLayout(null);

        jPanel1.setBorder(new javax.swing.border.TitledBorder("Sound Alarm when:"));
        alarm1RadioButton.setText("Any storm visible at my location");
        alarm1RadioButton.setToolTipText("Alarm will activate when any storm is above the horizon at the home location");
        alarmConditionsButtonGroup.add(alarm1RadioButton);
        jPanel1.add(alarm1RadioButton);
        alarm1RadioButton.setBounds(20, 30, 300, 23);

        alarm2RadioButton.setText("Mutually visible with DX or stations in list");
        alarm2RadioButton.setToolTipText("Alarm will sound when storms are above the horizon for home location and DX location as well as any stations in the Station List");
        alarmConditionsButtonGroup.add(alarm2RadioButton);
        jPanel1.add(alarm2RadioButton);
        alarm2RadioButton.setBounds(20, 60, 310, 23);

        alarm3RadioButton.setText("Mutually visible with stations in my list only");
        alarm3RadioButton.setToolTipText("Alarm will sound when storm is above the horizon for the home location and any stations in the Station list.");
        alarmConditionsButtonGroup.add(alarm3RadioButton);
        jPanel1.add(alarm3RadioButton);
        alarm3RadioButton.setBounds(20, 90, 300, 23);

        alarm4RadioButton.setSelected(true);
        alarm4RadioButton.setText("Mutually visible with DX location only");
        alarm4RadioButton.setToolTipText("Alarm will activate when any storm is above the horizon for both the Home and DX locations");
        alarmConditionsButtonGroup.add(alarm4RadioButton);
        jPanel1.add(alarm4RadioButton);
        alarm4RadioButton.setBounds(20, 120, 300, 23);

        getContentPane().add(jPanel1);
        jPanel1.setBounds(10, 10, 340, 160);

        jButton1.setText("OK");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        getContentPane().add(jButton1);
        jButton1.setBounds(120, 310, 120, 25);

        jCheckBox1.setSelected(true);
        jCheckBox1.setText("Activate alarm");
        getContentPane().add(jCheckBox1);
        jCheckBox1.setBounds(30, 190, 190, 23);

        pack();
    }//GEN-END:initComponents

    /** "OK" button was clicked.  Save settings to disk */
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        // Save alarm Settings
        saveAlarmSettings();
        this.setVisible(false);
    }//GEN-LAST:event_jButton1ActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        this.dispose();
    }//GEN-LAST:event_exitForm
    
   
    
    /**
     * This method handles all action events in this class.  When a user changes
     * the state of the buttons, the alarm is updated according.
     * @param e The ActionEvent to be handled by this method
     */    
    public void actionPerformed(java.awt.event.ActionEvent e) {
        Object source = e.getSource();
       
         if (source == alarm1RadioButton) {
            alarmCondition = ANY_VISIBILE_STORMS;
        }
        else if (source == alarm2RadioButton) {
            alarmCondition = DX_OR_STATIONS;
        }
        else if (source == alarm3RadioButton) {
            alarmCondition = STATIONS_ONLY;
        }
        else if (source == alarm4RadioButton) {
            alarmCondition = DX_ONLY;
        }
        else if (source == jCheckBox1) {
            if (jCheckBox1.isSelected()){
                alarmType = ALARM_ON;
                alarm1RadioButton.setEnabled(true);
                alarm2RadioButton.setEnabled(true);
                alarm3RadioButton.setEnabled(true);
                alarm4RadioButton.setEnabled(true);
            }
            else { 
                alarmType = ALARM_OFF;
                alarm1RadioButton.setEnabled(false);
                alarm2RadioButton.setEnabled(false);
                alarm3RadioButton.setEnabled(false);
                alarm4RadioButton.setEnabled(false);
            }
        }
        this.saveAlarmSettings();
        MAIN_PROGRAM.updateAlarmLabel();
    }
    /**
     * Reads the current alarm settings (from disk) and returns the alarm condition
     * @return  the alarm condition
     */    
    public int getAlarmCondition() {
        readAlarmSettings();
        return alarmCondition;
    }
    /**
     * Reads the current Alarm Type (from disk) and returns it.
     * @return  the alarm type
     */    
    public int getAlarmType() {
        readAlarmSettings();
        return alarmType;
    }
    private void readAlarmSettings()  {
        
        char c = (char)0;
        int i = 0;
        String ac = "";
        String at= "";
        
        
        try {
            FileReader in = new FileReader("alarmsettings.dat");
            while (c != (char)-1){
                try {
                    c = (char)in.read();
                    if (c == '\n') {
                        i++;
                    }
                    else {
                        switch (i) {
                            case (0): {ac = ac + c; break;}
                            case (1): {at = at + c; break;}
                            
                        }
                    }
                }
                catch (IOException e) {}
            }
            alarmCondition = new Integer(ac).intValue();
            alarmType = new Integer(at).intValue();
            
            
        }
        catch (FileNotFoundException e) {  //first time program starts
            alarmCondition = 4;
            alarmType = 2;
            saveAlarmSettings();
        }
        switch (alarmCondition) {
            case (ANY_VISIBILE_STORMS) : {alarm1RadioButton.setSelected(true); break;}
            case (DX_OR_STATIONS) : {alarm2RadioButton.setSelected(true); break;}
            case (STATIONS_ONLY) : {alarm3RadioButton.setSelected(true); break;}
            case (DX_ONLY) : {alarm4RadioButton.setSelected(true); break;}
        }
        switch (alarmType) {
            case (ALARM_OFF) : {
                jCheckBox1.setSelected(false); 
                alarm1RadioButton.setEnabled(false);
                alarm2RadioButton.setEnabled(false);
                alarm3RadioButton.setEnabled(false);
                alarm4RadioButton.setEnabled(false);
                break;}
            case (ALARM_ON) : {
                jCheckBox1.setSelected(true); 
                alarm1RadioButton.setEnabled(true);
                alarm2RadioButton.setEnabled(true);
                alarm3RadioButton.setEnabled(true);
                alarm4RadioButton.setEnabled(true);
                break;
            }
            
        }
    }
    private void saveAlarmSettings() {
       String ac = new Integer(alarmCondition).toString();
       String at = new Integer(alarmType).toString();
       
        try {
            File f = new File("alarmsettings.dat");
            FileWriter out = new FileWriter(f);
            
                out.write(ac + '\n');
                out.write(at + '\n');
           
            out.close();
        }
        catch (IOException ioe) { 
            javax.swing.JOptionPane.showMessageDialog(this, "Error writing to file.  Alarm Settings will not be saved."); 
        }
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JRadioButton alarm1RadioButton;
    private javax.swing.JRadioButton alarm2RadioButton;
    private javax.swing.JRadioButton alarm3RadioButton;
    private javax.swing.JRadioButton alarm4RadioButton;
    private javax.swing.ButtonGroup alarmConditionsButtonGroup;
    private javax.swing.ButtonGroup alarmTypeButtonGroup;
    private javax.swing.JButton jButton1;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables

}
