/*
 * jMap.java
 *
 * Created on May 31, 2004, 1:44 AM
 */

/**
 *
 * @author  Andrew T. Flowers
 */


import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * This is the map display for the RainScatter program.  It uses extends a JLabel
 * to do custom painting (stations, paths, etc).
 */
public class JMap extends JLabel{
   
    private final RainScatter MAIN_PROGRAM =                   //this is used to refer 
            (RainScatter)RainScatter.getFrames()[0];   //to the main program
    
    
    Color homeColor = Color.YELLOW;
    Color dxColor = Color.WHITE;
    Color pathColor = Color.WHITE;
    Color echoTopColor = Color.YELLOW;
    Color maxReflectivityColor = Color.RED;
    Color midpointColor = Color.PINK;
    
    Point2D.Double homeLocation;
    Point2D.Double dxLocation;
    Point2D.Double scatLocation;
    
    Point2D.Double refLatLon;   //reference for top left pix of map
    
    double degreesPerPixelX;
    double degreesPerPixelY;
    
    Storm storm;
   // int dxx;  //dx location in pixels
  //  int dxy;
    
  //  int myx;  //home location in pixels
  //  int myy;
    
 //   int scatx;  //scattering lcation in pixels
 //   int scaty;

    int fishEyeRadiusInPixels = 0;  //radii in the horizontal direction
    int stormRadiusInPixels = 0;    //they will be "squished" vertically
    int heightRadiusInPixels = 0;   //by map aspect

    //new parameters
    int homeFishEyeRadiusInPixelsX;
    int dxFishEyeRadiusInPixelsX;
    int echoTopRadiusInPixelsX;
    int echoTopRadusInPixelsY;
    int heightRadiusInPixelsX;
    int heightRadiusInPixelsY;
    

    /** Creates a new instance of jMap */
    public JMap() {
        Cursor c = new java.awt.Cursor(java.awt.Cursor.CROSSHAIR_CURSOR);
        setCursor(c);
        dxLocation = new Point2D.Double();
        homeLocation = new Point2D.Double();
        scatLocation = new Point2D.Double();
    }

     public void setFisheyeRadius(int radius) {
         this.fishEyeRadiusInPixels = radius;
     }
     
     public void setStormTopRadius(int radius) {
         this.stormRadiusInPixels = radius;
     }
     
    public void setMaxRefRadius(int radius) {
        this.heightRadiusInPixels = radius;
    }
    
    /*Set home location in Lat/Lon pair */
    public void setHomeLocation(Point2D.Double homeLocation) {
        this.homeLocation = homeLocation;
    }
    
    /*Set DX location in Lat/Lon pair */
    public void setDXLocation(Point2D.Double dxLocation) {
        this.dxLocation = dxLocation;
    }
    
    /*Set scattering location in Lat/Lon pair */
    public void setScatLocation(Point2D.Double scatLocation) {
        this.scatLocation = scatLocation;
    }
    
    public void setRefLatLon(Point2D.Double refLatLon) {
        this.refLatLon = refLatLon;
        
    }
    
    public void setDegreesPerPixelX(double d) {
        this.degreesPerPixelX = d;
    }
    
    public void setDegreesPerPixelY(double d) {
        this.degreesPerPixelY = d;
    }
    
    public void setStorm(Storm storm) {
        this.storm = storm;
    }
    private Point getPixelFromLatLon(Point2D.Double latLon) {
        Point pixel = new Point();

        pixel.setLocation( (refLatLon.y - latLon.y) / degreesPerPixelY,
        (refLatLon.x - latLon.x) / degreesPerPixelX);
        
        //System.out.println(pixel);
        return pixel;
    }

    /**
     * Painting of the image starts here.
     * @param g
     */
    protected void paintComponent(Graphics g) {
       
        
        super.paintComponent(g);   //paint map in background
        Graphics2D g2d = (Graphics2D)g;
        
        if ((homeLocation != null) && (dxLocation != null) && (refLatLon != null)) {
            drawHomeLocation(g2d);
            drawDXLocation(g2d);
            drawPath(g2d);
            if (storm != null){
                if (MAIN_PROGRAM.isEchoTopSelected()){
                    g2d.setColor(this.echoTopColor);
                    drawCircle(new Point2D.Double(storm.getLat(),storm.getLon()), 
                                (int)(storm.getTop() * 1000), g2d);
                }
                if (MAIN_PROGRAM.isMaxReflectivitySelected()){
                    g2d.setColor(maxReflectivityColor);
                    drawCircle(new Point2D.Double(storm.getLat(),storm.getLon()), 
                                 (int)(storm.getHT() * 1000), g2d);
                }
            }
        }
        
        g.dispose();  //clean up
    }
    
    private void drawHomeLocation(Graphics2D g2d) {
        Point home = getPixelFromLatLon(homeLocation);
        g2d.setColor(homeColor);
        drawX(home, g2d);
    }
    private void drawDXLocation(Graphics2D g2d) {
        g2d.setStroke(new BasicStroke(2));
        Point dx = getPixelFromLatLon(dxLocation);
        g2d.setColor(dxColor);
        drawX(dx, g2d);
    }
    private void drawPath(Graphics2D g2d) {
        g2d.setStroke(new BasicStroke(2));
        g2d.setColor(pathColor);
        Point scat = getPixelFromLatLon(scatLocation);
        Point home = getPixelFromLatLon(homeLocation);
        Point dx = getPixelFromLatLon(dxLocation);
        g2d.drawLine(scat.x, scat.y, home.x, home.y);
        g2d.drawLine(scat.x, scat.y, dx.x, dx.y);
    }
    
    private void drawCircle(Point2D.Double latLon, int heightInFeet, Graphics2D g2d) {
        //create a bounding rectangle for the "circle", which will appear 
        //flattened on the screen

        int width;  //width of bounding rectangle of oval
        int height;  //height of bounding rectangle of oval
        
        double squish;   // flattening factor of the cirlce
        
        Point p = getPixelFromLatLon(latLon);
        
        /*Now we calculate the amount of "squish" or flattening of the circle
         at this latitude.  Do this by comparing distances in the horizontal and
         vertical directions on the map*/
        
        Point v1 = new Point(p.x, p.y - 1);
        Point v2 = new Point(p.x, p.y + 1);
        Point h1 = new Point(p.x - 1, p.y );
        Point h2 = new Point(p.x + 1, p.y);
        
        Point2D.Double v1d = getLatLon(v1.x,v1.y);
        Point2D.Double v2d = getLatLon(v2.x,v2.y);
        Point2D.Double h1d = getLatLon(h1.x,h1.y);
        Point2D.Double h2d = getLatLon(h2.x,h2.y);
        
        
        double distanceHoriz = radiansToMiles(getDistanceRadians(v1d.x, v1d.y, v2d.x, v2d.y));
        double distanceVert = radiansToMiles(getDistanceRadians(h1d.x, h1d.y, h2d.x, h2d.y));
        
        double milesPerPixelHoriz = (distanceHoriz/2) * 0.77;  //this is a fudge factor
        squish = distanceVert/distanceHoriz; 
        
        double radius = getStormRadiusInMiles(heightInFeet);
        width = 2 * (int)(radius/milesPerPixelHoriz);
        
        height = (int)(width * squish);

        g2d.drawOval(p.x-width/2,p.y-height/2, width, height);
        
    }
   
    private void drawX(Point p, Graphics2D g2d) {
       
        g2d.setStroke(new BasicStroke(2));
        g2d.drawLine((p.x)-3, (p.y)-3, (p.x)+3, (p.y)+3);  
        g2d.drawLine((p.x)-3, (p.y)+3, (p.x)+3, (p.y)-3);
    }

    /**
     * Converts distance in radians on the surface of the earth to miles
     * @param rad
     * @return
     */
    private static double radiansToMiles(double rad){
        double miles = (rad*180*60/Math.PI)*1.15077945;  //to statute miles
        return miles;
    }
    
    /**Returns distance in radians, inputs are in degrees
     *
     */
    private static double getDistanceRadians(double latitude1, double longitude1, double latitude2, double longitude2) {
        double d;
        double lat1 = Math.toRadians(latitude1);
        double lon1 = Math.toRadians(longitude1);
        double lat2 = Math.toRadians(latitude2);
        double lon2 = Math.toRadians(longitude2);
        
        d=2*Math.asin(Math.sqrt(Math.pow((Math.sin((lat1-lat2)/2)),2) + Math.cos(lat1)*Math.cos(lat2)*Math.pow((Math.sin((lon1-lon2)/2)),2)));
        
        return d;
    }
    
    /*returns Point2D.Double lat/lon pair for given x/y coords on image */
    private java.awt.geom.Point2D.Double getLatLon(int x, int y) {
        double lat;
        double lon;
        java.awt.geom.Point2D.Double latlon= new java.awt.geom.Point2D.Double();
        
        latlon.x = refLatLon.x - degreesPerPixelY * y;
        latlon.y = refLatLon.y - degreesPerPixelX * x;
        
        return latlon;
    }
    
       
    /** Gets radius in miles for given storm height in feet*/
    private double getStormRadiusInMiles(double heightFeet) {
        double radius =   20912696.6 * Math.acos(20912696.6/(20912696.6 + heightFeet)) / 5280;
        return radius;
    }
}
