/*
 * Rainscatter.java
 *
 *This program displays the latest NWS radar information to be used for calculating
 *rainscatter paths.  It provides distance and bearing information for two stations.
 * It requires java v. 1.4.2 or later and an internet connection to download the
 * images and storm tables.
 *
 * Created on May 16, 2004, 6:20 PM
 *1.00 Monday, 10 January 2004
 */

/**
 * @version 1.0 Monday, 9 January 2005
 * @author  Andrew T. Flowers K0SM
 */
import javax.swing.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import java.io.File;
import java.util.prefs.*;

/** Constructor for a Rainscatter object */
public class RainScatter extends javax.swing.JFrame implements java.awt.event.ActionListener {

    /**
     * The version RainScatter. Public b/c it is used in forming the FTP password.
     */
    public final String VERSION = "RainScatterV1.70";
    private Preferences root = Preferences.userRoot();
    final Preferences node = root.node("gui/rainscatter");
    double degreesPerPixelX;
    double degreesPerPixelY;
    double refDegreesLat;  // reference for top left pixel.  Set by world file
    double refDegreesLon;
    java.awt.geom.Point2D.Double refLatLon;
    //Strings for assembling the URL for
    //the radar image
    final String radarBeginString = "https://radar.weather.gov/ridge/RadarImg/";
    final String shortBaseRadar = "N0R";
    final String shortCompRadar = "NCR";
    final String longBaseRadar = "N0Z";
    final String shortVILRadar = "NVL";
    final String shortTopRadar = "NET";
    final String radarEndString = "_0.gif";
    final double SHORTBASE = 0.53; //Miles per pixel for the different radar
    final double SHORTCOMP = 0.53;  //settings
    final double LONGBASE = 0.82;
    //boolean mapClickedDx = false;    //whether or not locations were set by clicking the map
    // boolean mapClickedHome = false;
    // boolean autoUpdate = false;
    final int refx = 300;   //radar location in all images
    final int refy = 275;
    double milesPerPixel = 0.42;  //initial condition
    private java.awt.geom.Point2D.Double dxLocation;
    private java.awt.geom.Point2D.Double homeLocation;
    private java.awt.geom.Point2D.Double scatLocation;
    int homeX;  //home location, pixels
    int homeY;
    int dxrefx;   // dx location, initialized to my own location
    int dxrefy;
    int scatx;
    int scaty;
    /**
     * The home location grid square.  Public b/c used in forming FTP password
     */
    public String myGrid;   //public because StormTableReader uses it for
    // part of the FTP password
    String dxGrid;
    String oldDXGrid;
    int stormRadiusInPixels = 0;
    int stormRadiusInMiles = 0;
    int fishEyeRadiusInPixels = 0;
    int fishEyeRadiusInMiles = 0;
    int heightRadiusInPixels = 0;
    int heightRadiusInMiles = 0;
    private int updateInterval;
    private javax.swing.Timer updateTimer;
    private JMap map;
    private java.util.Vector stormTable;
    private java.util.Vector stationList;
    private StationsMap stationsMap;
    private StormTableReader stormTableReader;
    private StormViewer stormTableViewer;
    private AlarmSettingsWindow alarmSettingsWindow;
    private StationListWindow stationListWindow;
    private JToggleButton[] visibleStationsButtons = new JToggleButton[0];
    private java.util.Vector visibleStationsStormTables;
    private java.util.Vector visibleStationsVector;
    private java.util.Vector myStormTable;
    private java.util.Vector dxStormTable;
    ///Strings for the Radar selection combo box
    String[] radars = {
        "Birmingham, AL",
        "Fort Rucker, AL",
        "Huntsville, AL",
        "Mobile, AL",
        "Montgomery, AL",
        "Bethel, AK",
        "Biorka I., AK",
        "Kenai, AK",
        "King Salmon, AK",
        "Middleton I., AK",
        "Nome, AK",
        "Pedro Dome, AK",
        "Fort Smith, AR",
        "Little Rock, AR",
        "Flagstaff, AZ",
        "Phoenix, AZ",
        "Tucson, AZ",
        "Yuma, AZ",
        "Beale AFB, CA",
        "Edwards AFB, CA",
        "Eureka, CA",
        "Los Angeles, CA",
        "Sacramento, CA",
        "San Diego, CA",
        "San Francisco, CA",
        "San Joaquin Valley, CA",
        "Santa Ana Mtns, CA",
        "Vandenberg AFB, CA",
        "Denver, CO",
        "Grand Junction, CO",
        "Pueblo, CO",
        "Dover, DE",
        "Jacksonville, FL",
        "Key West, FL",
        "Melbourne, FL",
        "Miami, FL",
        "Panama City, FL",
        "Tallahassee, FL",
        "Tampa Bay, FL",
        "Atlanta, GA",
        "Moody AFB, GA",
        "Robins AFB, GA",
        "Kauai, HI",
        "Kohala, HI",
        "Honolulu, HI",
        "Molokai, HI",
        "Des Moines, IA",
        "Quad Cities, IA",
        "Boise, ID",
        "Pocatello, ID",
        "Chicago, IL",
        "Springfield, IL",
        "Evansville, IN",
        "Indianapolis, IN",
        "South Bend, IN",
        "Dodge City, KS",
        "Goodland, KS",
        "Topeka, KS",
        "Wichita, KS",
        "Fort Cambell, KY",
        "Jackson, KY",
        "Louisville, KY",
        "Paducah, KY",
        "Fort Polk, LA",
        "Lake Charles, LA",
        "New Orleans, LA",
        "Shreveport, LA",
        "Boston, MA",
        "Caribou, ME",
        "Portland, ME",
        "Detroit, MI",
        "Gaylord, MI",
        "Grand Rapids, MI",
        "Marquette, MI",
        "Duluth, MN",
        "Minneapolis, MN",
        "Kansas City, MO",
        "Saint Louis, MO",
        "Springfield, MO",
        "Columbus AFB, MS",
        "Jackson, MS",
        "Billings, MT",
        "Glascow, MT",
        "Great Falls, MT",
        "Missoula, MT",
        "Newport, NC",
        "Raleigh, NC",
        "Wilmington, NC",
        "Bismarck, ND",
        "Grand Forks, ND",
        "Minot, ND",
        "Hastings, NE",
        "North Platte, NE",
        "Omaha, NE",
        "Mt. Holly, NJ",
        "Albuquerque, NM",
        "Cannon AFB, NM",
        "Holloman AFB, NM",
        "Elko, NV",
        "Las Vegas, NV",
        "Reno, NV",
        "Albany, NY",
        "Binghamton, NY",
        "Buffalo, NY",
        "Montague, NY",
        "New York, NY",
        "Cleveland, OH",
        "Wilmington, OH",
        "Frederick, OK",
        "Oklahoma City, OK",
        "Tulsa, OK",
        "Vance AFB, OK",
        "Medford, OR",
        "Pendleton, OR",
        "Portland, OR",
        "Pittsburgh, PA",
        "State College, PA",
        "Charleston, SC",
        "Columbia, SC",
        "Greer, SC",
        "Aberdeen, SD",
        "Rapid City, SD",
        "Sioux Falls, SD",
        "Knoxville, TN",
        "Memphis, TN",
        "Nashville, TN",
        "Amarillo, TX",
        "Austin, TX",
        "Brownsville, TX",
        "Corpus Christi, TX",
        "Dallas, TX",
        "Dyess AFB, TX",
        "El Paso, TX",
        "Houston, TX",
        "Laughlin AFB, TX",
        "Lubbock, TX",
        "Midland, TX",
        "San Angelo, TX",
        "San Antonio, TX",
        "Salt Lake City, UT",
        "Blacksburg, VA",
        "Sterling, VA",
        "Wakefield, VA",
        "Burlington, VT",
        "Seattle, WA",
        "Spokane, WA",
        "Green Bay, WI",
        "La Crosse, WI",
        "Milwaukee, WI",
        "Charleston, WV",
        "Cheyenne, WY",
        "Riverton, WY",
        "San Juan, PR",
        "Andersen AFB, Guam"
    };
    String currentRadar;
    String currentRadarName;
    String currentRadarView;
    String currentURLString = "https://radar.weather.gov/ridge/RadarImg/N0R/BUF_N0R_0.gif";
    String currentTopoURL = "https://radar.weather.gov/ridge/Overlays/Topo/Short/BUF_Topo_Short.jpg";
    String currentCountyURL = "https://radar.weather.gov/ridge/Overlays/County/Short/BUF_County_Short.gif";
    String currentCityURL = "https://radar.weather.gov/ridge/Overlays/Cities/Short/BUF_City_Short.gif";
    String currentHighwayURL = "https://radar.weather.gov/ridge/Overlays/Highways/Short/BUF_Highways_Short.gif";
    String currentRiverURL = "https://radar.weather.gov/ridge/Overlays/Rivers/Short/BUF_Rivers_Short.gif";
    String currentWarningURL = "https://radar.weather.gov/ridge/Warnings/Short/BUF__Warnings_0.gif";
    String currentLegendURL = "https:/radar.weather.gov/ridge/Legend/N0R/Short/BUF_N0R_Legend_0.gif";
    Image currentImage = null;
    java.awt.image.BufferedImage currentBufferedImage;
    //String tempGrid;       //used for storing the grid for mouse entered/exit methods

    /** Creates new form RainScatter */
    public RainScatter() {
        stormTable = new java.util.Vector();
        stationList = new java.util.Vector();
        homeLocation = new java.awt.geom.Point2D.Double();
        dxLocation = new java.awt.geom.Point2D.Double();
        scatLocation = new java.awt.geom.Point2D.Double();

        //       degreesPerPixelX = node.getDouble("degreesPerPixelX", 0.00982652143998578); //default for Buffalo NY
        //       degreesPerPixelY = node.getDouble("degreesPerPixelY", 0.00982652143998578); //shortrange base
        //       refDegreesLat = node.getDouble("refDegreesLat",45.646381447532);
        //       refDegreesLon = node.getDouble("refDegreesLon", 81.6800426829945);

        loadSettings();

        initComponents();



        initRadarButtons();
        loadCheckBoxSettings();

        loadRadarList();

        //setSize(800, 600);

        homeGridField.setText(myGrid);
        dxGridField.setText(dxGrid);
        stormTableReader = new StormTableReader();
        stormTableViewer = new StormViewer();
        alarmSettingsWindow = new AlarmSettingsWindow();
        stationListWindow = new StationListWindow();
        stationList = stationListWindow.getStations();
        updateTimer = new javax.swing.Timer(5 * 60000, new java.awt.event.ActionListener() {

            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateTimerActionPerformed(evt);
            }
        });

        if (autoCheckBox.isSelected()) {
            updateTimer.start();

            if (alarmSettingsWindow.getAlarmType() == alarmSettingsWindow.ALARM_ON) {  // if alarms are enabled
                alarmLabel.setText("Alarm: ON");
            }
        }
        if (loadMapCheckBox.isSelected()) {
            updateMap();
        }
        if (loadStormsCheckBox.isSelected()) {
            updateStormsButton.doClick();
        }


    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        radarButtonGroup1 = new javax.swing.ButtonGroup();
        radarSelectionGroup = new javax.swing.ButtonGroup();
        fishEyeButtonGroup = new javax.swing.ButtonGroup();
        buttonGroup1 = new javax.swing.ButtonGroup();
        statusLabel = new javax.swing.JLabel();
        guiButtonGroup = new javax.swing.ButtonGroup();
        echoTopsButton = new javax.swing.JRadioButton();
        quickStartMenuItem = new javax.swing.JMenuItem();
        bearingPanel = new javax.swing.JPanel();
        visibleStationsButtonGroup = new javax.swing.ButtonGroup();
        mapPanel = new javax.swing.JPanel();
        jLayeredPane1 = new javax.swing.JLayeredPane();
        map = new JMap();
        map.setBounds(0, 0, 600, 550);
        jLayeredPane1.add(map, new Integer(5));

        map.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                mapMouseReleased(evt);
            }
        });
        map.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseDragged(java.awt.event.MouseEvent evt) {
                mapMouseDragged(evt);
            }
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                mapMouseMoved(evt);
            }
        });
        topoMap = new javax.swing.JLabel();
        countyMap = new javax.swing.JLabel();
        cityMap = new javax.swing.JLabel();
        highwayMap = new javax.swing.JLabel();
        riverMap = new javax.swing.JLabel();
        warningMap = new javax.swing.JLabel();
        warningMap.setCursor(new java.awt.Cursor(java.awt.Cursor.CROSSHAIR_CURSOR));
        legendMap = new javax.swing.JLabel();
        legendMap.setCursor(new java.awt.Cursor(java.awt.Cursor.CROSSHAIR_CURSOR));
        stationsMap = new StationsMap(this);
        stationsMap.setText("");

        stationsMap.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        stationsMap.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        stationsMap.setBounds(0, 0, 600, 550);
        jLayeredPane1.add(stationsMap, new Integer(6));
        myStationPanel = new javax.swing.JPanel();
        jLabel5 = new javax.swing.JLabel();
        jTextField4 = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        jTextField3 = new javax.swing.JTextField();
        setHomeButton = new javax.swing.JToggleButton();
        jLabel6 = new javax.swing.JLabel();
        homeGridField = new javax.swing.JTextField();
        radarPanel = new javax.swing.JPanel();
        radarOverlayPanel = new javax.swing.JPanel();
        topoCheckBox = new javax.swing.JCheckBox();
        countiesCheckBox = new javax.swing.JCheckBox();
        citiesCheckBox = new javax.swing.JCheckBox();
        highwaysCheckBox = new javax.swing.JCheckBox();
        riversCheckBox = new javax.swing.JCheckBox();
        warningsCheckBox = new javax.swing.JCheckBox();
        stationsCheckBox = new javax.swing.JCheckBox();
        legendCheckBox = new javax.swing.JCheckBox();
        gridLabel = new javax.swing.JLabel();
        radarSelectionPanel = new javax.swing.JPanel();
        radarSelectionComboBox = new javax.swing.JComboBox();
        radarSelectionComboBox.setSelectedItem((Object)currentRadarName);
        updateMapButton = new javax.swing.JButton();
        updateStormsButton = new javax.swing.JButton();
        viewStormsButton = new javax.swing.JButton();
        radarUpdatePanel = new javax.swing.JPanel();
        autoCheckBox = new javax.swing.JCheckBox();
        jSpinner1 = new javax.swing.JSpinner(new javax.swing.SpinnerNumberModel(5, 1, 30, 1));
        ;
        jLabel8 = new javax.swing.JLabel();
        radarViewPanel = new javax.swing.JPanel();
        baseButton = new javax.swing.JRadioButton();
        compositeButton = new javax.swing.JRadioButton();
        longrangeButton = new javax.swing.JRadioButton();
        imageProgressBar = new javax.swing.JProgressBar();
        stormTableProgressBar = new javax.swing.JProgressBar();
        jLabel17 = new javax.swing.JLabel();
        alarmLabel = new javax.swing.JLabel();
        pathPanel = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jLabel16 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        otherStationPanel = new javax.swing.JPanel();
        jLabel10 = new javax.swing.JLabel();
        jTextField9 = new javax.swing.JTextField();
        jLabel9 = new javax.swing.JLabel();
        jTextField8 = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        setDXButton = new javax.swing.JToggleButton();
        dxGridField = new javax.swing.JTextField();
        jPanel1 = new javax.swing.JPanel();
        visibleStationsPanel = new javax.swing.JPanel();
        visibleStormsPanel = new javax.swing.JPanel();
        jMenuBar1 = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        exitMenuItem = new javax.swing.JMenuItem();
        settingsMenu = new javax.swing.JMenu();
        radarListMenuItem = new javax.swing.JMenuItem();
        stationListMenuItem = new javax.swing.JMenuItem();
        AlarmSettingsMenuItem = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        startupMenu = new javax.swing.JMenu();
        loadMapCheckBox = new javax.swing.JCheckBoxMenuItem();
        loadStormsCheckBox = new javax.swing.JCheckBoxMenuItem();
        drawMenu = new javax.swing.JMenu();
        footprintMenu = new javax.swing.JMenu();
        echoTopsCheckBoxMenuItem = new javax.swing.JCheckBoxMenuItem();
        heightCheckBoxMenuItem = new javax.swing.JCheckBoxMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        jumpCheckBoxMenuItem = new javax.swing.JCheckBoxMenuItem();
        helpMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();

        radarButtonGroup1.add(echoTopsButton);
        echoTopsButton.setText("Echo Tops");
        echoTopsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                echoTopsButtonActionPerformed(evt);
            }
        });

        quickStartMenuItem.setText("Quick start...");
        quickStartMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                quickStartMenuItemActionPerformed(evt);
            }
        });

        bearingPanel.setLayout(new java.awt.BorderLayout());

        setTitle("RainScatter by K0SM");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        mapPanel.setBackground(new java.awt.Color(0, 0, 0));
        mapPanel.setMaximumSize(new java.awt.Dimension(600, 550));
        mapPanel.setMinimumSize(new java.awt.Dimension(600, 550));
        mapPanel.setPreferredSize(new java.awt.Dimension(600, 550));
        mapPanel.setLayout(new javax.swing.BoxLayout(mapPanel, javax.swing.BoxLayout.LINE_AXIS));

        jLayeredPane1.setMaximumSize(new java.awt.Dimension(600, 550));
        jLayeredPane1.setMinimumSize(new java.awt.Dimension(600, 550));

        topoMap.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        topoMap.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        jLayeredPane1.add(topoMap);
        topoMap.setBounds(0, 0, 600, 550);
        jLayeredPane1.add(countyMap);
        countyMap.setBounds(0, 0, 600, 550);
        jLayeredPane1.setLayer(countyMap, 2);
        jLayeredPane1.add(cityMap);
        cityMap.setBounds(0, 0, 600, 550);
        jLayeredPane1.setLayer(cityMap, 5);

        highwayMap.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        highwayMap.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        jLayeredPane1.add(highwayMap);
        highwayMap.setBounds(0, 0, 600, 550);
        jLayeredPane1.setLayer(highwayMap, 4);

        riverMap.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        riverMap.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        jLayeredPane1.add(riverMap);
        riverMap.setBounds(0, 0, 600, 550);
        jLayeredPane1.setLayer(riverMap, 3);

        warningMap.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        warningMap.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        jLayeredPane1.add(warningMap);
        warningMap.setBounds(0, 0, 600, 550);
        jLayeredPane1.setLayer(warningMap, 6);

        legendMap.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        legendMap.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        jLayeredPane1.add(legendMap);
        legendMap.setBounds(0, 0, 600, 550);
        jLayeredPane1.setLayer(legendMap, 7);

        mapPanel.add(jLayeredPane1);

        myStationPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("My station"));
        myStationPanel.setFocusable(false);

        jLabel5.setText("Heading:");

        jTextField4.setEditable(false);
        jTextField4.setColumns(4);
        jTextField4.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextField4.setToolTipText("Beam heading for home station");
        jTextField4.setAutoscrolls(false);
        jTextField4.setDisabledTextColor(new java.awt.Color(255, 255, 255));
        jTextField4.setFocusable(false);

        jLabel4.setText("Distance:");

        jTextField3.setEditable(false);
        jTextField3.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextField3.setToolTipText("Distance from home location to storm");
        jTextField3.setDisabledTextColor(new java.awt.Color(255, 255, 255));
        jTextField3.setFocusable(false);
        jTextField3.setPreferredSize(new java.awt.Dimension(57, 21));

        setHomeButton.setText("Set Home Loc");
        setHomeButton.setToolTipText("Click to change the home location");
        setHomeButton.setNextFocusableComponent(setDXButton);
        setHomeButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                setHomeButtonMouseClicked(evt);
            }
        });

        jLabel6.setText("mi");

        homeGridField.setEditable(false);
        homeGridField.setColumns(6);
        homeGridField.setFont(new java.awt.Font("Dialog", 1, 18)); // NOI18N
        homeGridField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        homeGridField.setToolTipText("Grid square of home location");
        homeGridField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                homeGridFieldFocusLost(evt);
            }
        });
        homeGridField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                homeGridFieldActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout myStationPanelLayout = new javax.swing.GroupLayout(myStationPanel);
        myStationPanel.setLayout(myStationPanelLayout);
        myStationPanelLayout.setHorizontalGroup(
            myStationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(myStationPanelLayout.createSequentialGroup()
                .addGap(4, 4, 4)
                .addComponent(jLabel5, javax.swing.GroupLayout.PREFERRED_SIZE, 70, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(10, 10, 10)
                .addComponent(jTextField4, javax.swing.GroupLayout.PREFERRED_SIZE, 50, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addGroup(myStationPanelLayout.createSequentialGroup()
                .addGap(4, 4, 4)
                .addComponent(jLabel4, javax.swing.GroupLayout.PREFERRED_SIZE, 60, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(20, 20, 20)
                .addComponent(jTextField3, javax.swing.GroupLayout.PREFERRED_SIZE, 50, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(10, 10, 10)
                .addComponent(jLabel6, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addGroup(myStationPanelLayout.createSequentialGroup()
                .addGap(34, 34, 34)
                .addComponent(homeGridField, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addGroup(myStationPanelLayout.createSequentialGroup()
                .addGap(14, 14, 14)
                .addComponent(setHomeButton, javax.swing.GroupLayout.PREFERRED_SIZE, 140, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        myStationPanelLayout.setVerticalGroup(
            myStationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(myStationPanelLayout.createSequentialGroup()
                .addGap(4, 4, 4)
                .addGroup(myStationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel5, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jTextField4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(10, 10, 10)
                .addGroup(myStationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jTextField3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel6, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(9, 9, 9)
                .addComponent(homeGridField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(0, 0, 0)
                .addComponent(setHomeButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        radarPanel.setFocusable(false);

        radarOverlayPanel.setLayout(new javax.swing.BoxLayout(radarOverlayPanel, javax.swing.BoxLayout.X_AXIS));

        topoCheckBox.setSelected(true);
        topoCheckBox.setText("Topo");
        topoCheckBox.addActionListener(this);
        topoCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                topoCheckBoxActionPerformed(evt);
            }
        });
        radarOverlayPanel.add(topoCheckBox);

        countiesCheckBox.setSelected(true);
        countiesCheckBox.setText("Counties");
        countiesCheckBox.addActionListener(this);
        countiesCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                countiesCheckBoxActionPerformed(evt);
            }
        });
        radarOverlayPanel.add(countiesCheckBox);

        citiesCheckBox.setSelected(true);
        citiesCheckBox.setText("Cities");
        citiesCheckBox.addActionListener(this);
        citiesCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                citiesCheckBoxActionPerformed(evt);
            }
        });
        radarOverlayPanel.add(citiesCheckBox);

        highwaysCheckBox.setSelected(true);
        highwaysCheckBox.setText("Highways");
        highwaysCheckBox.addActionListener(this);
        highwaysCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                highwaysCheckBoxActionPerformed(evt);
            }
        });
        radarOverlayPanel.add(highwaysCheckBox);

        riversCheckBox.setSelected(true);
        riversCheckBox.setText("Rivers");
        riversCheckBox.addActionListener(this);
        riversCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                riversCheckBoxActionPerformed(evt);
            }
        });
        radarOverlayPanel.add(riversCheckBox);

        warningsCheckBox.setSelected(true);
        warningsCheckBox.setText("Warnings");
        warningsCheckBox.addActionListener(this);
        warningsCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                warningsCheckBoxActionPerformed(evt);
            }
        });
        radarOverlayPanel.add(warningsCheckBox);

        stationsCheckBox.setSelected(true);
        stationsCheckBox.setText("Stations");
        stationsCheckBox.addActionListener(this);
        stationsCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stationsCheckBoxActionPerformed(evt);
            }
        });
        radarOverlayPanel.add(stationsCheckBox);

        legendCheckBox.setSelected(true);
        legendCheckBox.setText("Legend");
        legendCheckBox.addActionListener(this);
        legendCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                legendCheckBoxActionPerformed(evt);
            }
        });
        radarOverlayPanel.add(legendCheckBox);

        gridLabel.setFont(new java.awt.Font("MS Sans Serif", 0, 8)); // NOI18N
        gridLabel.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        gridLabel.setToolTipText("Grid square of cursor in map window");
        radarOverlayPanel.add(gridLabel);

        radarSelectionComboBox.setToolTipText("Select radar from list");
        radarSelectionComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                radarSelectionComboBoxActionPerformed(evt);
            }
        });

        updateMapButton.setText("Update Map");
        updateMapButton.setToolTipText("Updates the image now");
        updateMapButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateMapButtonActionPerformed(evt);
            }
        });

        updateStormsButton.setText("Update Storms");
        updateStormsButton.setToolTipText("Updates the Master Storm Table and looks for visible stations");
        updateStormsButton.setActionCommand("Updata Storms");
        updateStormsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateStormsButtonActionPerformed(evt);
            }
        });

        viewStormsButton.setText("View All Storms");
        viewStormsButton.setToolTipText("Click to view all storms identified by radars in list");
        viewStormsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                viewStormsButtonActionPerformed(evt);
            }
        });

        radarUpdatePanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Update"));
        radarUpdatePanel.setFocusable(false);

        autoCheckBox.setText("Auto");
        autoCheckBox.setToolTipText("Activates automatic updates");
        autoCheckBox.addActionListener(this);
        autoCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                autoCheckBoxActionPerformed(evt);
            }
        });

        jSpinner1.setToolTipText("change to select the update interval in minutes");
        jSpinner1.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinner1StateChanged(evt);
            }
        });

        jLabel8.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel8.setText("min");

        javax.swing.GroupLayout radarUpdatePanelLayout = new javax.swing.GroupLayout(radarUpdatePanel);
        radarUpdatePanel.setLayout(radarUpdatePanelLayout);
        radarUpdatePanelLayout.setHorizontalGroup(
            radarUpdatePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(radarUpdatePanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(autoCheckBox, javax.swing.GroupLayout.PREFERRED_SIZE, 60, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSpinner1, javax.swing.GroupLayout.PREFERRED_SIZE, 40, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel8, javax.swing.GroupLayout.PREFERRED_SIZE, 40, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        radarUpdatePanelLayout.setVerticalGroup(
            radarUpdatePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(radarUpdatePanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(radarUpdatePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(autoCheckBox)
                    .addComponent(jSpinner1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel8, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        javax.swing.GroupLayout radarSelectionPanelLayout = new javax.swing.GroupLayout(radarSelectionPanel);
        radarSelectionPanel.setLayout(radarSelectionPanelLayout);
        radarSelectionPanelLayout.setHorizontalGroup(
            radarSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(radarSelectionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(radarSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(radarSelectionComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 170, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(radarUpdatePanel, javax.swing.GroupLayout.PREFERRED_SIZE, 162, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(radarSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(viewStormsButton, javax.swing.GroupLayout.DEFAULT_SIZE, 164, Short.MAX_VALUE)
                    .addComponent(updateStormsButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(updateMapButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        radarSelectionPanelLayout.setVerticalGroup(
            radarSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(radarSelectionPanelLayout.createSequentialGroup()
                .addGroup(radarSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(radarSelectionComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(updateMapButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(radarSelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(radarSelectionPanelLayout.createSequentialGroup()
                        .addComponent(updateStormsButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(viewStormsButton))
                    .addComponent(radarUpdatePanel, javax.swing.GroupLayout.PREFERRED_SIZE, 50, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        radarViewPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("View"));
        radarViewPanel.setFocusable(false);
        radarViewPanel.setLayout(new javax.swing.BoxLayout(radarViewPanel, javax.swing.BoxLayout.Y_AXIS));

        radarButtonGroup1.add(baseButton);
        baseButton.setSelected(true);
        baseButton.setText("Base");
        baseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                baseButtonActionPerformed(evt);
            }
        });
        radarViewPanel.add(baseButton);

        radarButtonGroup1.add(compositeButton);
        compositeButton.setText("Composite ");
        compositeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                compositeButtonActionPerformed(evt);
            }
        });
        radarViewPanel.add(compositeButton);

        radarButtonGroup1.add(longrangeButton);
        longrangeButton.setText("Longrange");
        longrangeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                longrangeButtonActionPerformed(evt);
            }
        });
        radarViewPanel.add(longrangeButton);

        imageProgressBar.setFocusable(false);
        imageProgressBar.setPreferredSize(new java.awt.Dimension(150, 10));

        stormTableProgressBar.setPreferredSize(new java.awt.Dimension(150, 10));

        jLabel17.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel17.setText("Alarm:");

        alarmLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        alarmLabel.setText("OFF");

        javax.swing.GroupLayout radarPanelLayout = new javax.swing.GroupLayout(radarPanel);
        radarPanel.setLayout(radarPanelLayout);
        radarPanelLayout.setHorizontalGroup(
            radarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, radarPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(radarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(radarOverlayPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(radarPanelLayout.createSequentialGroup()
                        .addComponent(radarSelectionPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(radarViewPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(radarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(stormTableProgressBar, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, radarPanelLayout.createSequentialGroup()
                                .addComponent(jLabel17, javax.swing.GroupLayout.PREFERRED_SIZE, 50, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(alarmLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 40, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(imageProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE))
                        .addContainerGap())))
        );
        radarPanelLayout.setVerticalGroup(
            radarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(radarPanelLayout.createSequentialGroup()
                .addComponent(radarOverlayPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(radarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(radarViewPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(radarPanelLayout.createSequentialGroup()
                        .addComponent(imageProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(0, 0, 0)
                        .addComponent(stormTableProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(radarPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel17)
                            .addComponent(alarmLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(radarSelectionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        pathPanel.setLayout(new java.awt.GridLayout(1, 6));

        jLabel2.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel2.setText("Target:");
        jLabel2.setToolTipText("Grid square of scattering point");
        pathPanel.add(jLabel2);

        jLabel16.setFont(new java.awt.Font("MS Sans Serif", 1, 12)); // NOI18N
        jLabel16.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel16.setText("Not set");
        jLabel16.setToolTipText("Grid square of scattering point");
        pathPanel.add(jLabel16);

        jLabel11.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel11.setText("Path:");
        jLabel11.setToolTipText("Total distance home station to storm to DX location");
        pathPanel.add(jLabel11);

        jLabel12.setFont(new java.awt.Font("MS Sans Serif", 1, 11)); // NOI18N
        jLabel12.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel12.setText("0");
        jLabel12.setToolTipText("Total distance home station to storm to DX location");
        pathPanel.add(jLabel12);

        jLabel3.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel3.setText("Angle:");
        jLabel3.setToolTipText("Scattering angle of path between DX and home locations");
        pathPanel.add(jLabel3);

        jLabel14.setFont(new java.awt.Font("MS Sans Serif", 1, 11)); // NOI18N
        jLabel14.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel14.setText("0");
        jLabel14.setToolTipText("Scattering angle of path between DX and home locations");
        pathPanel.add(jLabel14);

        otherStationPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Other station"));
        otherStationPanel.setFocusable(false);

        jLabel10.setText("Heading:");

        jTextField9.setEditable(false);
        jTextField9.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextField9.setToolTipText("Beam heading for DX station");
        jTextField9.setDisabledTextColor(new java.awt.Color(255, 255, 255));
        jTextField9.setFocusable(false);
        jTextField9.setPreferredSize(new java.awt.Dimension(38, 21));

        jLabel9.setText("Distance:");

        jTextField8.setEditable(false);
        jTextField8.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
        jTextField8.setToolTipText("Distance from DX location to storm");
        jTextField8.setDisabledTextColor(new java.awt.Color(255, 255, 255));
        jTextField8.setFocusable(false);
        jTextField8.setPreferredSize(new java.awt.Dimension(57, 21));

        jLabel7.setText("mi");

        setDXButton.setText("Set DX Loc");
        setDXButton.setToolTipText("click to change the DX location");
        setDXButton.setNextFocusableComponent(setHomeButton);
        setDXButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                setDXButtonMouseClicked(evt);
            }
        });

        dxGridField.setEditable(false);
        dxGridField.setColumns(6);
        dxGridField.setFont(new java.awt.Font("Dialog", 1, 18)); // NOI18N
        dxGridField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        dxGridField.setToolTipText("Grid square of DX location");
        dxGridField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dxGridFieldActionPerformed(evt);
            }
        });
        dxGridField.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                dxGridFieldFocusLost(evt);
            }
        });

        javax.swing.GroupLayout otherStationPanelLayout = new javax.swing.GroupLayout(otherStationPanel);
        otherStationPanel.setLayout(otherStationPanelLayout);
        otherStationPanelLayout.setHorizontalGroup(
            otherStationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(otherStationPanelLayout.createSequentialGroup()
                .addGap(14, 14, 14)
                .addComponent(jLabel10, javax.swing.GroupLayout.PREFERRED_SIZE, 60, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(10, 10, 10)
                .addComponent(jTextField9, javax.swing.GroupLayout.PREFERRED_SIZE, 50, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addGroup(otherStationPanelLayout.createSequentialGroup()
                .addGap(14, 14, 14)
                .addComponent(jLabel9, javax.swing.GroupLayout.PREFERRED_SIZE, 60, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(10, 10, 10)
                .addComponent(jTextField8, javax.swing.GroupLayout.PREFERRED_SIZE, 50, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(10, 10, 10)
                .addComponent(jLabel7, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addGroup(otherStationPanelLayout.createSequentialGroup()
                .addGap(34, 34, 34)
                .addComponent(dxGridField, javax.swing.GroupLayout.PREFERRED_SIZE, 100, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addGroup(otherStationPanelLayout.createSequentialGroup()
                .addGap(14, 14, 14)
                .addComponent(setDXButton, javax.swing.GroupLayout.PREFERRED_SIZE, 140, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        otherStationPanelLayout.setVerticalGroup(
            otherStationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(otherStationPanelLayout.createSequentialGroup()
                .addGap(4, 4, 4)
                .addGroup(otherStationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel10)
                    .addComponent(jTextField9, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(9, 9, 9)
                .addGroup(otherStationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel9, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jTextField8, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel7, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(9, 9, 9)
                .addComponent(dxGridField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(0, 0, 0)
                .addComponent(setDXButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Visible Stations"));
        jPanel1.setLayout(new java.awt.BorderLayout());

        visibleStationsPanel.setLayout(new java.awt.GridLayout(15, 1));
        jPanel1.add(visibleStationsPanel, java.awt.BorderLayout.WEST);

        visibleStormsPanel.setPreferredSize(new java.awt.Dimension(165, 500));
        visibleStormsPanel.setLayout(new java.awt.BorderLayout());
        jPanel1.add(visibleStormsPanel, java.awt.BorderLayout.CENTER);

        fileMenu.setText("File");

        exitMenuItem.setText("Exit");
        exitMenuItem.setToolTipText("Terminates the program");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });
        fileMenu.add(exitMenuItem);

        jMenuBar1.add(fileMenu);

        settingsMenu.setText("Settings");

        radarListMenuItem.setText("Radar List");
        radarListMenuItem.setToolTipText("Change the active list of radars");
        radarListMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                radarListMenuItemActionPerformed(evt);
            }
        });
        settingsMenu.add(radarListMenuItem);

        stationListMenuItem.setText("Station List");
        stationListMenuItem.setToolTipText("Edit the the list of stations for the storm alarm");
        stationListMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stationListMenuItemActionPerformed(evt);
            }
        });
        settingsMenu.add(stationListMenuItem);

        AlarmSettingsMenuItem.setText("Alarm Settings");
        AlarmSettingsMenuItem.setToolTipText("Change the alarm settings");
        AlarmSettingsMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                AlarmSettingsMenuItemActionPerformed(evt);
            }
        });
        settingsMenu.add(AlarmSettingsMenuItem);
        settingsMenu.add(jSeparator2);

        startupMenu.setText("Startup");

        loadMapCheckBox.setText("Load map");
        startupMenu.add(loadMapCheckBox);

        loadStormsCheckBox.setText("Load Storms");
        startupMenu.add(loadStormsCheckBox);

        settingsMenu.add(startupMenu);

        jMenuBar1.add(settingsMenu);

        drawMenu.setText("Draw");

        footprintMenu.setText("Footprints");
        footprintMenu.setToolTipText("Draw visible range of storm from\n top and/or point of max reflectivity");

        echoTopsCheckBoxMenuItem.setSelected(true);
        echoTopsCheckBoxMenuItem.setText("Echo top");
        echoTopsCheckBoxMenuItem.setToolTipText("");
        footprintMenu.add(echoTopsCheckBoxMenuItem);

        heightCheckBoxMenuItem.setSelected(true);
        heightCheckBoxMenuItem.setText("Max reflectivity");
        footprintMenu.add(heightCheckBoxMenuItem);

        drawMenu.add(footprintMenu);
        drawMenu.add(jSeparator4);

        jumpCheckBoxMenuItem.setText("Jump to radar");
        jumpCheckBoxMenuItem.setToolTipText("Jump to radar map when \nstorm selected in table");
        drawMenu.add(jumpCheckBoxMenuItem);

        jMenuBar1.add(drawMenu);

        helpMenu.setText("Help");

        aboutMenuItem.setText("About");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        helpMenu.add(aboutMenuItem);

        jMenuBar1.add(helpMenu);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(mapPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(radarPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(pathPanel, javax.swing.GroupLayout.PREFERRED_SIZE, 358, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(myStationPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(otherStationPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGap(0, 8, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(mapPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(radarPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addGap(0, 0, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(myStationPanel, javax.swing.GroupLayout.PREFERRED_SIZE, 150, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(otherStationPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(pathPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, 496, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void echoTopsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_echoTopsButtonActionPerformed
        if (echoTopsButton.isSelected()) {
            currentRadarView = shortTopRadar;

            updateMap();
        }
    }//GEN-LAST:event_echoTopsButtonActionPerformed

    private void postInit() {
        initComponents();


        initRadarButtons();
        loadCheckBoxSettings();

        loadRadarList();

        //setSize(800, 600);

        homeGridField.setText(myGrid);
        dxGridField.setText(dxGrid);
        stormTableReader = new StormTableReader();
        stormTableViewer = new StormViewer();
        alarmSettingsWindow = new AlarmSettingsWindow();
        stationListWindow = new StationListWindow();
        stationList = stationListWindow.getStations();
        updateTimer = new javax.swing.Timer(5 * 60000, new java.awt.event.ActionListener() {

            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateTimerActionPerformed(evt);
            }
        });

        if (autoCheckBox.isSelected()) {
            updateTimer.start();

            if (alarmSettingsWindow.getAlarmType() == alarmSettingsWindow.ALARM_ON) {  // if alarms are enabled
                alarmLabel.setText("Alarm: ON");
            }
        }
        if (loadMapCheckBox.isSelected()) {
            updateMap();
        }
        if (loadStormsCheckBox.isSelected()) {
            updateStormsButton.doClick();
        }



    }

    /** This opens the online radar using "address" as the String representation
     * of the URL.  It then sets jLabel to this imageIcon and sets jScrollPane1 to
     * display it.  This is done in a separate thread because it is called from action
     * listners.  (Otherwise the GUI would hang until the image was downoaded.) Painting
     * the stations and path is done on the main thread afterward.  */
    private void openRadarImage(String address) {
        stationList = stationListWindow.getStations();

        mapPanel.setVisible(true);
        imageProgressBar.setIndeterminate(true);
        imageProgressBar.setToolTipText("Downloading radar image");
        Toolkit toolkit = Toolkit.getDefaultToolkit();

        /**Gets slow network traffic onto it's own thread. Swingworker
         * is an Object designed to do exactly that, and then exectute
         * some code after the thread is finished.
         *http://java.sun.com/docs/books/tutorial/uiswing/misc/threads.html#SwingWorker
         */
        final SwingWorker worker = new SwingWorker() {

            public Object construct() {
                Integer n = new Integer(0);  //required for SwingWorker
                map.setVisible(false);
                topoMap.setVisible(false);
                countyMap.setVisible(false);
                cityMap.setVisible(false);
                highwayMap.setVisible(false);
                riverMap.setVisible(false);
                warningMap.setVisible(false);
                stationsMap.setVisible(false);
                legendMap.setVisible(false);
                for (int i = 0; i < stationList.size(); i++) {
                    stationsMap.setStationList(stationList);
                }


                try {
                    cityMap.setIcon(new javax.swing.ImageIcon(fetchimage(currentCityURL)));
                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println(e.getMessage());
                }
                try {
                    countyMap.setIcon(new javax.swing.ImageIcon(fetchimage(currentCountyURL)));
                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println(e.getMessage());
                }
                try {
                    topoMap.setIcon(new javax.swing.ImageIcon(fetchimage(currentTopoURL)));
                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println(e.getMessage());
                }
                try {
                    highwayMap.setIcon(new javax.swing.ImageIcon(fetchimage(currentHighwayURL)));
                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println(e.getMessage());
                }
                try {
                    riverMap.setIcon(new javax.swing.ImageIcon(fetchimage(currentRiverURL)));
                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println(e.getMessage());
                }
                try {
                    warningMap.setIcon(new javax.swing.ImageIcon(fetchimage(currentWarningURL)));
                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println(e.getMessage());
                }
                try {
                    legendMap.setIcon(new javax.swing.ImageIcon(fetchimage(currentLegendURL)));
                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println(e.getMessage());
                }
                try {
                    map.setIcon(new javax.swing.ImageIcon(fetchimage(currentURLString)));
                } catch (Exception e) {
                    e.printStackTrace();
                    System.out.println(e.getMessage());
                }
                try {
                    //now get the world file for this radar
                    //world file uses gfw extension for the corresponding gif
                    String worldURLString = currentURLString.replaceAll("gif", "gfw");

                    //No WF for ET or VIL, so use Shortrange-base radar's 
                    worldURLString = worldURLString.replaceAll("NET", "N0R");
                    worldURLString = worldURLString.replaceAll("NVL", "N0R");

                    URL worldURL = new URL(worldURLString);
                    System.out.println(worldURL);
                    InputStream inStream = worldURL.openStream();
                    BufferedReader in = new BufferedReader(new InputStreamReader(inStream));
                    String line;
                    int lineNumber = 0;
                    while ((line = in.readLine()) != null) {
                        switch (lineNumber) {
                            case (0): {
                                degreesPerPixelX = Double.parseDouble(line);
                                lineNumber++;
                                break;
                            }
                            case (1): {
                                lineNumber++;
                                break;
                            }
                            case (2): {
                                lineNumber++;
                                break;
                            }
                            case (3): {

                                degreesPerPixelY = -(Double.parseDouble(line));
                                lineNumber++;
                                break;
                            }
                            case (4): {
                                refDegreesLon = -(Double.parseDouble(line));
                                lineNumber++;
                                break;
                            }
                            case (5): {
                                refDegreesLat = Double.parseDouble(line);
                                lineNumber++;
                                break;
                            }

                        }
                    }
                    refLatLon = new java.awt.geom.Point2D.Double(refDegreesLat, refDegreesLon);
                    //set milesPerPixel based on lat and deg/pixel
                    java.awt.geom.Point2D.Double p1 = getLatLon(0, 225);
                    java.awt.geom.Point2D.Double p2 = getLatLon(600, 225);

                    double d = getDistance(p1.y, p1.x, p2.y, p2.x);

                    System.out.println("Distance across: " + radiansToMiles(d));
                    System.out.println("MPP Y: " + 69.172 * degreesPerPixelY);

                    //milesPerPixelY = 69.172 * degreesPerPixelY;
                } catch (MalformedURLException exc) {
                    javax.swing.JOptionPane.showMessageDialog(mapPanel, "Bad URL when forming address.  Please report this error");
                } catch (IOException exc) {
                    javax.swing.JOptionPane.showMessageDialog(mapPanel, "Unable to download image.  Server may not be responding.  Check your internet connection and try again.");
                }
                if (topoCheckBox.isSelected()) {
                    topoMap.setVisible(true);
                }
                if (citiesCheckBox.isSelected()) {
                    cityMap.setVisible(true);
                }
                if (countiesCheckBox.isSelected()) {
                    countyMap.setVisible(true);
                }
                if (riversCheckBox.isSelected()) {
                    riverMap.setVisible(true);
                }
                if (highwaysCheckBox.isSelected()) {
                    highwayMap.setVisible(true);
                }
                if (warningsCheckBox.isSelected()) {
                    warningMap.setVisible(true);
                }
                if (stationsCheckBox.isSelected()) {

                    stationsMap.setVisible(true);

                }
                if (legendCheckBox.isSelected()) {
                    legendMap.setVisible(true);
                }
                map.setVisible(true);
                mapPanel.validate();

                return n;   //not used, but required of this class
            }
            //after thread is finished, back on EventDisptach thread (b/c is has to be)

            public void finished() {
                mapPanel.setVisible(true);
                //  try {
                Point home = getPixelFromGrid(homeGridField.getText());
                Point dx = getPixelFromGrid(dxGridField.getText());
                Point scat = getPixelFromGrid(getGrid(scatLocation.x, scatLocation.y));

                homeX = home.x;
                homeY = home.y;
                dxrefx = dx.x;
                dxrefy = dx.y;
                scatx = scat.x;
                scaty = scat.y;
                // }
                // catch (Exception e) {
                //e.printStackTrace();  //thrown when program first starts b/c no home or dx location set
                // }
                paintLocations();
                imageProgressBar.setIndeterminate(false);
                imageProgressBar.setToolTipText(null);
            }
        };
        worker.start();     //start the thread
    }
    private void quickStartMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_quickStartMenuItemActionPerformed
        QuickStartWindow qsw = new QuickStartWindow();
        qsw.setVisible(true);
    }//GEN-LAST:event_quickStartMenuItemActionPerformed

    private void AlarmSettingsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_AlarmSettingsMenuItemActionPerformed
        alarmSettingsWindow.setVisible(true);
    }//GEN-LAST:event_AlarmSettingsMenuItemActionPerformed

    private void stationListMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stationListMenuItemActionPerformed
        stationListWindow = new StationListWindow();
        stationListWindow.setVisible(true);

    }//GEN-LAST:event_stationListMenuItemActionPerformed

    private void mapMouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mapMouseMoved
        gridLabel.setText(getGrid(refDegreesLat - degreesPerPixelY * evt.getY(),
                refDegreesLon - degreesPerPixelX * evt.getX()));
        //System.out.println(evt.getX() + "  " + evt.getY());
    }//GEN-LAST:event_mapMouseMoved

    private void mapMouseDragged(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mapMouseDragged
        stormRadiusInMiles = 0;
        stormRadiusInPixels = 0;
        heightRadiusInMiles = 0;
        heightRadiusInPixels = 0;
        if (setDXButton.isSelected()) {        //set dx location
            dxrefx = evt.getX();
            dxrefy = evt.getY();

            dxGridField.setText(getGrid(dxrefx, dxrefy));
            paintLocations();
            setDXButton.setSelected(false);
            dxGridField.setEditable(false);

        } else if (setHomeButton.isSelected()) {     //set home location
            homeX = evt.getX();
            homeY = evt.getY();
            //mapClickedHome = true;
            homeGridField.setText(getGrid(homeX, homeY));
            paintLocations();
            setHomeButton.setSelected(false);
            //            homeSet = true;
            homeGridField.setEditable(false);
        }
        //       if (dxSet && homeSet) {
        displayInfo(evt.getX(), evt.getY());
        paintLocations();

        //        }
    }//GEN-LAST:event_mapMouseDragged

    private void mapMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_mapMouseReleased
        stormRadiusInMiles = 0;
        stormRadiusInPixels = 0;
        heightRadiusInMiles = 0;
        heightRadiusInPixels = 0;
        if (setDXButton.isSelected()) {        //set dx location
            dxrefx = evt.getX();
            dxrefy = evt.getY();
            dxLocation = getLatLon(dxrefx, dxrefy);
            dxGrid = getGrid(dxrefx, dxrefy);
            dxGridField.setText(dxGrid);
            findPaths();
            displayInfo(scatx, scaty);
            paintLocations();
            setDXButton.setSelected(false);
            //            dxSet = true;
            dxGridField.setEditable(false);
        } else if (setHomeButton.isSelected()) {     //set home location
            homeX = evt.getX();
            homeY = evt.getY();
            // mapClickedHome = true;
            homeLocation = getLatLon(homeX, homeY);
            myGrid = getGrid(homeX, homeY);
            homeGridField.setText(myGrid);
            findPaths();
            displayInfo(scatx, scaty);
            paintLocations();
            setHomeButton.setSelected(false);
            homeGridField.setEditable(false);
        }

        scatx = evt.getX();
        scaty = evt.getY();
        scatLocation = getLatLon(scatx, scaty);
        displayInfo(evt.getX(), evt.getY());
        paintLocations();

    }//GEN-LAST:event_mapMouseReleased

    /**
     * Updates the radar list in radarSelectionComboBox.  Accessed by RadarListFrame
     * @param strs the string[] of radar names
     */
    public void changeRadarList(String[] strs) {

        DefaultComboBoxModel model = new DefaultComboBoxModel();
        for (int i = 0; i < strs.length; i++) {
            model.addElement(strs[i]);
        }
        model.setSelectedItem((Object) strs[0]);
        radarSelectionComboBox.setModel(model);
    }
    private void radarListMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_radarListMenuItemActionPerformed
        RadarListFrame rf = new RadarListFrame(radars);
        rf.setSize(250, 600);
        rf.setVisible(true);

    }//GEN-LAST:event_radarListMenuItemActionPerformed

    private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
        javax.swing.JOptionPane.showMessageDialog(this,
                "<html><H1><center>RainScatter</H1>"
                + "<br>"
                + "<H3>"
                + VERSION
                + "<br>"
                + "by Andrew T. Flowers, K0SM "
                + "<br>"
                + "<br>"
                + "(aflowers@frontiernet.net)</h3><br>"
                + "http://www.frontiernet.net/~aflowers/rainscatter"
                + "</Center</html>");
    }//GEN-LAST:event_aboutMenuItemActionPerformed

    private void updateTimerActionPerformed(java.awt.event.ActionEvent evt) {
        updateMap();          //update the map
        updateStormsButton.doClick();   //download storm data from FTP site
    }

    /* The user clicked away from the dxridField.  This does the same thing as
     * hitting the enter key. */
    private void dxGridFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_dxGridFieldFocusLost
        dxGridField.setText(dxGrid);
        dxGridField.setEditable(false);
        setDXButton.setSelected(false);
    }//GEN-LAST:event_dxGridFieldFocusLost

    /** The user clicked away from the homeGridField.  This does the same thing as
     * hitting the enter key. */
    private void homeGridFieldFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_homeGridFieldFocusLost
        homeGridField.setText(myGrid);
        homeGridField.setEditable(false);
        setHomeButton.setSelected(false);
    }//GEN-LAST:event_homeGridFieldFocusLost

    /** The user pressed the enter key while dxGridField had focus.  This sets
     *the dx location to this grid */
    private void dxGridFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_dxGridFieldActionPerformed

        boolean goodGrid = false;
        String string = dxGridField.getText();

        //check to see if it is a callsign in the stations list.
        //if so, get grid and use that:
        for (int i = 0; i < stationList.size(); i++) {
            Station station = (Station) (stationList.get(i));
            if (station.getCall().equalsIgnoreCase(string)) {
                string = station.getGrid();
                break;
            }

        }
        //correct to AAnnaa format
        if (string.length() == 6) {
            String strBegin = string.substring(0, 2).toUpperCase();
            String strMiddle = string.substring(2, 4);
            String strEnd = string.substring(4, 6).toLowerCase();
            string = strBegin + strMiddle + strEnd;
        }
        if (isGrid(string)) {
            goodGrid = true;
            Point p = getPixelFromGrid(string);
            dxrefx = p.x;
            dxrefy = p.y;
            dxLocation = getLatLon(dxrefx, dxrefy);
            dxGridField.setEditable(false);
            setDXButton.setSelected(false);
        } else {
            string = "";
            // dxSet = false;
        }
        dxGridField.setText(string);
        if (goodGrid) {
            //            dxSet = true;
            dxGrid = getGrid(dxrefx, dxrefy);
        }
        displayInfo(scatx, scaty);

        paintLocations();
        setDXButton.requestFocus();
        if (!(evt.getActionCommand() == null || evt.getActionCommand().equals("noFindPaths"))) {
            if (goodGrid) {
                findPaths();       //if the event comes from one of the visible stations buttons
            }
        }                      //don't call findPaths again
    }//GEN-LAST:event_dxGridFieldActionPerformed

    /** The user pressed the enter key while homeGridField had focus.  This sets
     *the home location to this grid */
    private void homeGridFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_homeGridFieldActionPerformed
        boolean goodGrid = false;
        String string = homeGridField.getText();

        //correct to AAnnaa format
        if (string.length() == 6) {
            String strBegin = string.substring(0, 2).toUpperCase();
            String strMiddle = string.substring(2, 4);
            String strEnd = string.substring(4, 6).toLowerCase();
            string = strBegin + strMiddle + strEnd;
        }
        if (isGrid(string)) {
            goodGrid = true;
            Point p = getPixelFromGrid(string);
            homeX = p.x;
            homeY = p.y;
            homeLocation = getLatLon(homeX, homeY);
            homeGridField.setEditable(false);
            setHomeButton.setSelected(false);
        } else {
            string = "";
            //            homeSet = false;
        }
        homeGridField.setText(string);
        if (goodGrid) {
            //            homeSet = true;
            myGrid = getGrid(homeX, homeY);
        }
        displayInfo(scatx, scaty);
        paintLocations();
        setHomeButton.requestFocus();
        setHomeButton.setSelected(false);
        homeGridField.setEditable(false);
        if (!(evt.getActionCommand() == null || evt.getActionCommand().equals("noFindPaths"))) {
            if (goodGrid) {
                findPaths();       //if the event comes from one of the visible stations buttons
            }
        }                      //don't call findPaths again
    }//GEN-LAST:event_homeGridFieldActionPerformed

    /** The "Set Home Loc" button is clicked */
    private void setHomeButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_setHomeButtonMouseClicked

        setDXButton.setSelected(false);
        if (setHomeButton.isSelected() == true) {
            homeGridField.setEditable(!homeGridField.isEditable());
            homeGridField.requestFocus();
            homeGridField.selectAll();
        } else {
            homeGridField.setEditable(false);
        }
    }//GEN-LAST:event_setHomeButtonMouseClicked

    /** The "Set DX Loc" button is clicked */
    private void setDXButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_setDXButtonMouseClicked
        setHomeButton.setSelected(false);
        if (setDXButton.isSelected() == true) {
            dxGridField.setEditable(!dxGridField.isEditable());
            dxGridField.requestFocus();
            dxGridField.selectAll();
        } else {
            dxGridField.setEditable(false);
        }
    }//GEN-LAST:event_setDXButtonMouseClicked

   /** Rescales the graphics to ensure that pixels scale with true distance */
    private void reScale() {
        fishEyeRadiusInPixels = (int) (((double) fishEyeRadiusInMiles) / milesPerPixel);
        fishEyeRadiusInPixels = (int) (fishEyeRadiusInPixels * 0.89);
        stormRadiusInPixels = (int) (((double) stormRadiusInMiles) / milesPerPixel);
        stormRadiusInPixels = (int) (stormRadiusInPixels * 0.89);
        heightRadiusInPixels = (int) (((double) heightRadiusInMiles) / milesPerPixel);
        heightRadiusInPixels = (int) (heightRadiusInPixels * 0.89);
    }

    /** "Open Local File" is clicked.  Useful for those who want to download, save, and run up a hill with their
     * laptops and microwave gear.    This brings up a file chooser in which the user selects an image file and
     * the radar view.  Selecting the radar view tells the program what grid map to apply to the image.*/
    /** The mouse is moved over the radar map .  This method calculates
     * if either of the set location buttons are selected it will update the
     *appropriate text box with the grid of the mouse cursor.
     */
    /** The "Exit" option is chosen from the File menu.  Closes the program.*/
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
        saveSettings();
        System.exit(0);
    }//GEN-LAST:event_exitMenuItemActionPerformed

    /** The "X" button is chosen from the upper right corner of the form. Closes program  */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        saveSettings();
        System.exit(0);
    }//GEN-LAST:event_exitForm

    private void viewStormsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_viewStormsButtonActionPerformed
        viewStormTable();
        stormTableViewer.setTitle("Storm Table");
    }//GEN-LAST:event_viewStormsButtonActionPerformed

    private void updateStormsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_updateStormsButtonActionPerformed

        //Logging into FTP server could cause GUI to hang, so move it off Main Dispatch Thread
        final SwingWorker downloadWorker = new SwingWorker() {

            public Object construct() {
                Object obj = new Object(); //not used in this class, but necessary for SwingWorker override
                java.util.Vector vect = new java.util.Vector();
                ComboBoxModel model = radarSelectionComboBox.getModel();
                for (int i = 0; i < model.getSize(); i++) {
                    vect.add(model.getElementAt(i));

                }
                stormTableReader.setRadarList(vect);
                stormTableReader.downloadData();
                return obj;
            }
        };
        downloadWorker.start();
    }//GEN-LAST:event_updateStormsButtonActionPerformed

    private void updateMapButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_updateMapButtonActionPerformed
        updateMap();
    }//GEN-LAST:event_updateMapButtonActionPerformed

    private void radarSelectionComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_radarSelectionComboBoxActionPerformed
        /** A radar is selected from the combo box */

        String str = (String) radarSelectionComboBox.getSelectedItem();
        String line = "";
        currentRadarName = str;
        try {
            File radardata = new File("radarlocations.txt");
            RandomAccessFile raf = new RandomAccessFile(radardata, "r");
            //find str in file

            boolean found = false;
            while (!found) {
                line = raf.readLine();

                java.util.StringTokenizer st = new java.util.StringTokenizer(line, "|");
                String[] data = new String[st.countTokens()];
                int count = st.countTokens();
                for (int i = 0; i < count; i++) {
                    data[i] = st.nextToken();
                }
                if (data[0].equals(str)) {
                    currentRadar = data[1].substring(1, 4);

                    found = true;
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

        updateMap();
    }//GEN-LAST:event_radarSelectionComboBoxActionPerformed

    /* The spinner for update interval was changed.  When this happens,
     * the timer is restarted. */
    private void jSpinner1StateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinner1StateChanged
        Number n = (Number) jSpinner1.getValue();
        updateInterval = n.intValue() * 60000;
        updateTimer.setInitialDelay(10000);
        updateTimer.setDelay(updateInterval);
        if (autoCheckBox.isSelected()) {
            updateTimer.restart();
        }
    }//GEN-LAST:event_jSpinner1StateChanged

    /** The longrange radiobutton is selected */
    private void longrangeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_longrangeButtonActionPerformed
        if (longrangeButton.isSelected()) {
            currentRadarView = longBaseRadar;
            //milesPerPixel = LONGBASE;
            //reScale();
            updateMap();
        }
    }//GEN-LAST:event_longrangeButtonActionPerformed

    /** The composite radiobutton is selected */
    private void compositeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_compositeButtonActionPerformed
        if (compositeButton.isSelected()) {
            currentRadarView = shortCompRadar;
            updateMap();
        }
    }//GEN-LAST:event_compositeButtonActionPerformed

    /** The base radiobutton is selected */
    private void baseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_baseButtonActionPerformed
        if (baseButton.isSelected()) {
            currentRadarView = shortBaseRadar;
            //milesPerPixel = LONGBASE;
            // reScale();
            updateMap();
        }
    }//GEN-LAST:event_baseButtonActionPerformed

    private void topoCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_topoCheckBoxActionPerformed
        topoMap.setVisible(topoCheckBox.isSelected());
    }//GEN-LAST:event_topoCheckBoxActionPerformed

    private void countiesCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_countiesCheckBoxActionPerformed
        countyMap.setVisible(countiesCheckBox.isSelected());
    }//GEN-LAST:event_countiesCheckBoxActionPerformed

    private void citiesCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_citiesCheckBoxActionPerformed
        cityMap.setVisible(citiesCheckBox.isSelected());
    }//GEN-LAST:event_citiesCheckBoxActionPerformed

    private void highwaysCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_highwaysCheckBoxActionPerformed
        highwayMap.setVisible(highwaysCheckBox.isSelected());
    }//GEN-LAST:event_highwaysCheckBoxActionPerformed

    private void riversCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_riversCheckBoxActionPerformed
        riverMap.setVisible(riversCheckBox.isSelected());
    }//GEN-LAST:event_riversCheckBoxActionPerformed

    private void warningsCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_warningsCheckBoxActionPerformed
        warningMap.setVisible(warningsCheckBox.isSelected());
    }//GEN-LAST:event_warningsCheckBoxActionPerformed

    private void stationsCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stationsCheckBoxActionPerformed
        stationsMap.setVisible(stationsCheckBox.isSelected());
    }//GEN-LAST:event_stationsCheckBoxActionPerformed

    private void legendCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_legendCheckBoxActionPerformed
         legendMap.setVisible(legendCheckBox.isSelected());
    }//GEN-LAST:event_legendCheckBoxActionPerformed

    private void autoCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_autoCheckBoxActionPerformed
        if (autoCheckBox.isSelected()) {
                updateTimer.start();
                if (alarmSettingsWindow.getAlarmType() == alarmSettingsWindow.ALARM_ON) {  // if alarms are enabled
                    alarmLabel.setText("ON");
                }
            } else {
                updateTimer.stop();
                alarmLabel.setText("OFF");

            }
    }//GEN-LAST:event_autoCheckBoxActionPerformed

    /*sets the status label when downloading files */
    public void setStatusLabel(String string, Color color) {
        System.out.println("status: " + string);
        statusLabel.setForeground(color);
        statusLabel.setText(string);
    }

    public void setStormTableProgressBarIndeterminate(boolean b) {
        if (b) {
            stormTableProgressBar.setIndeterminate(true);
        } else {
            stormTableProgressBar.setIndeterminate(false);
        }
    }

    /** Updates all of the text boxes and labels with the current heading information*/
    private void displayInfo(int x, int y) {
        //        if (dxSet && homeSet) {
        int myd;  //distance from home location to scattering point
        int dxd;  // distance from DX location to scattering point
        double myheading;
        double dxheading;
        double scatAngle;
        scatLocation = getLatLon(x, y);
        jLabel16.setText((getGrid(x, y)));

        scatx = x;
        scaty = y;

        //now calculate distances for both
        myd = (int) radiansToMiles(getDistance(homeLocation.x, homeLocation.y, scatLocation.x, scatLocation.y));
        dxd = (int) radiansToMiles(getDistance(dxLocation.x, dxLocation.y, scatLocation.x, scatLocation.y));

        //calculate headings for both
        myheading = getHeading(homeLocation.x, homeLocation.y, scatLocation.x, scatLocation.y);
        dxheading = getHeading(dxLocation.x, dxLocation.y, scatLocation.x, scatLocation.y);

        scatAngle = Math.abs(getHeading(scatLocation.x, scatLocation.y, homeLocation.x, homeLocation.y)
                - getHeading(scatLocation.x, scatLocation.y, dxLocation.x, dxLocation.y));

        if (scatAngle > 180) {
            scatAngle = 360.0 - scatAngle;
        }

        //truncate to one decimal place
        myheading = (Math.round(myheading * 10)) / 10.0;
        dxheading = (Math.round(dxheading * 10)) / 10.0;
        scatAngle = (Math.round(scatAngle * 10)) / 10.0;

        Integer mydist = new Integer(myd);
        Integer dxdist = new Integer(dxd);
        Integer totaldistance = new Integer(dxd + myd);
        Double myhead = new Double(myheading);
        Double dxhead = new Double(dxheading);
        Double scatterangle = new Double(scatAngle);

        jTextField3.setText(mydist.toString());
        jTextField4.setText(myhead.toString());
        jTextField8.setText(dxdist.toString());
        jTextField9.setText(dxhead.toString());
        jLabel12.setText(totaldistance.toString() + " mi");
        jLabel14.setText(scatterangle.toString() + "\u00b0");
        //       }
    }

    /** This is a more efficient method of calculating exponents in an integer world. It
     * is used for distance calulation using the image pixels as distane units. */
    private int powInt(int a, int b) {
        int i;
        int total = 1;
        for (i = 0; i < b; i++) {
            total = total * a;
        }
        return total;
    }

    /** assembles the entire URL using the current settings and tells the program to
     *  open the location. */
    private void updateMap() {
        currentURLString = radarBeginString + currentRadarView + "/"
                + currentRadar + "_" + currentRadarView + "_0.gif";

        currentTopoURL = "https://radar.weather.gov/ridge/Overlays/Topo/";
        currentCountyURL = "https://radar.weather.gov/ridge/Overlays/County/";
        currentCityURL = "https://radar.weather.gov/ridge/Overlays/Cities/";
        currentHighwayURL = "https://radar.weather.gov/ridge/Overlays/Highways/";
        currentRiverURL = "https://radar.weather.gov/ridge/Overlays/Rivers/";
        currentWarningURL = "https://radar.weather.gov/ridge/Warnings/";
        currentLegendURL = "https://radar.weather.gov/ridge/Legend/";

        if (longrangeButton.isSelected()) {
            currentTopoURL = currentTopoURL + "Long/" + currentRadar + "_Topo_Long.jpg";
            currentCountyURL = currentCountyURL + "Long/" + currentRadar + "_County_Long.gif";
            currentCityURL = currentCityURL + "Long/" + currentRadar + "_City_Long.gif";
            currentHighwayURL = currentHighwayURL + "Long/" + currentRadar + "_Highways_Long.gif";
            currentRiverURL = currentRiverURL + "Long/" + currentRadar + "_Rivers_Long.gif";
            currentWarningURL = currentWarningURL + "Long/" + currentRadar + "_Warnings_0.gif";
            currentLegendURL = currentLegendURL + currentRadarView + "/"
                    + currentRadar + "_" + currentRadarView
                    + "_Legend_0.gif";
        } else {
            currentTopoURL = currentTopoURL + "Short/" + currentRadar + "_Topo_Short.jpg";
            currentCountyURL = currentCountyURL + "Short/" + currentRadar + "_County_Short.gif";
            currentCityURL = currentCityURL + "Short/" + currentRadar + "_City_Short.gif";
            currentHighwayURL = currentHighwayURL + "Short/" + currentRadar + "_Highways_Short.gif";
            currentRiverURL = currentRiverURL + "Short/" + currentRadar + "_Rivers_Short.gif";
            currentWarningURL = currentWarningURL + "Short/" + currentRadar + "_Warnings_0.gif";
            currentLegendURL = currentLegendURL + currentRadarView + "/"
                    + currentRadar + "_" + currentRadarView
                    + "_Legend_0.gif";
        }

        openRadarImage(currentURLString);

    }
    /*
    private void makeMapCorrection(String radName) {
    String str = (String)radarSelectionComboBox.getSelectedItem();
    if (str == null) {
    str = "Buffalo, NY";  //default
    }
    String line = "";
    currentRadarName = str;


    try{
    File radardata = new File("radarlocations.txt");
    RandomAccessFile raf = new RandomAccessFile(radardata, "r");
    //find str in file

    boolean found = false;
    while (!found) {
    line = raf.readLine();
    java.util.StringTokenizer st = new java.util.StringTokenizer(line,"|");
    String[] data = new String[st.countTokens()];
    int count = st.countTokens();
    for (int i = 0; i < count; i++) {
    data[i] = st.nextToken();
    }
    if (data[0].equals(str)) {
    currentRadar = data[1];
    //  radlat = new Double(data[2]).doubleValue();
    //   radlon = new Double(data[3]).doubleValue();
    found = true;
    }
    }
    }
    catch (IOException e) {e.printStackTrace();}
    }
     */

    /** The "Open Online Radar" item was choses from the File menu.  It displays
     * radar settings panel and sets the proper scaling for the image.
     */
    /** Get the contents of a URL and return it as an image.  Called by OpenRadarImage. */
    private Image fetchimage(String address) throws MalformedURLException, IOException {
        Image a;

        URL url = new URL(address);
        //  currentBufferedImage = ImageIO.read(url);

        System.out.println(url);
        currentImage = createImage((java.awt.image.ImageProducer) url.getContent());


        a = currentImage;
        return a;
    }

    private void paintLocations() {

        // map.setCoords(homeX, homeY, dxrefx, dxrefy, scatx, scaty, refx, refy, fishEyeRadiusInPixels, stormRadiusInPixels, heightRadiusInPixels);
        // map.setDXLocation(dxrefx, dxrefy);
        // map.setHomeLocation(homeX, homeY);
        // map.setScatLocation(scatx, scaty);
        map.setRefLatLon(refLatLon);
        map.setDegreesPerPixelX(degreesPerPixelX);
        map.setDegreesPerPixelY(degreesPerPixelY);

        System.out.println("ScatLocation:  " + scatLocation);
        map.setDXLocation(dxLocation);

        map.setHomeLocation(homeLocation);
        map.setScatLocation(scatLocation);
        /*
        System.out.println(refLatLon);
        System.out.println(degreesPerPixelX);
        System.out.println(degreesPerPixelY);
        System.out.println(dxLocation);
        System.out.println(homeLocation);
        System.out.println(scatLocation);

         */
        //map.setFisheyeRadius(fishEyeRadiusInPixels);
        // map.setStormTopRadius(stormRadiusInPixels);
        // map.setMaxRefRadius(heightRadiusInPixels);

        map.repaint();
    }

    /**Returns distance in radians, inputs are in degrees
     *
     */
    private double getDistance(double latitude1, double longitude1, double latitude2, double longitude2) {  //works
        double d;
        double lat1 = Math.toRadians(latitude1);
        double lon1 = Math.toRadians(longitude1);
        double lat2 = Math.toRadians(latitude2);
        double lon2 = Math.toRadians(longitude2);

        d = 2 * Math.asin(Math.sqrt(Math.pow((Math.sin((lat1 - lat2) / 2)), 2) + Math.cos(lat1) * Math.cos(lat2) * Math.pow((Math.sin((lon1 - lon2) / 2)), 2)));

        return d;
    }

    /**
     *lat/lon in degrees, rng in miles, az in degrees
     */
    private java.awt.geom.Point2D.Double getLatLon(double refLat, double refLon, double az, double rng) {
        double lat;
        double lon;
        double azd;
        double rngd;
        rngd = rng * Math.PI / (180 * 60);
        azd = Math.toRadians(az);
        java.awt.geom.Point2D.Double p = new java.awt.geom.Point2D.Double();
        lat = Math.asin(Math.sin(Math.toRadians(refLat)) * Math.cos(rngd) + Math.cos(Math.toRadians(refLat)) * Math.sin(rngd) * Math.cos(azd));
        if (Math.cos(lat) == 0) {
            lon = -Math.toRadians(refLon);
        } else {
            lon = ((-Math.toRadians(refLon) - Math.asin(Math.sin(azd) * Math.sin(rngd) / Math.cos(lat)) + Math.PI)
                    % (2 * Math.PI)) - Math.PI;
        }
        p.x = Math.toDegrees(lat);
        p.y = Math.toDegrees(lon);

        return p;
    }

    /**
     * Heading from lat/lon1 to lat lon2 by GC. Input is in degrees, returns degrees */
    private double getHeading(double latitude1, double longitude1, double latitude2, double longitude2) {
        double tc1;  //initial course from point 1
        double d = getDistance(latitude1, longitude1, latitude2, longitude2);
        double lat1 = Math.toRadians(latitude1);
        double lon1 = Math.toRadians(longitude1);
        double lat2 = Math.toRadians(latitude2);
        double lon2 = Math.toRadians(longitude2);

        if (Math.sin(lon2 - lon1) < 0) {
            tc1 = Math.acos((Math.sin(lat2) - Math.sin(lat1) * Math.cos(d)) / (Math.sin(d) * Math.cos(lat1)));
        } else {
            tc1 = 2 * Math.PI - Math.acos((Math.sin(lat2) - Math.sin(lat1) * Math.cos(d)) / (Math.sin(d) * Math.cos(lat1)));
        }
        tc1 = Math.toDegrees(tc1);
        return tc1;
    }

    private String getGrid(double lat, double lon) {
        //field = 20 degrees lon, 10 deg lat
        //grid = 2 degrees lon, 1 deg lat
        //subsquare = 2/24 lon, 1/24 lat;
        //'A' = (char)65
        //'0' (zero)  = (char)(48)
        //West is positive
        String str = "";
        char g1;
        char g2;
        char g3;
        char g4;
        char g5;
        char g6;
        g1 = (char) (Math.floor((-lon + 180) / 20.0) + 65);  //x
        g2 = (char) (Math.floor(lat / 10.0) + 74);  //x

        g3 = (char) (Math.floor(((-lon + 180) % 20) / 2) + 48); //x
        g4 = (char) (Math.floor((lat + 90) % 10) + 48);   //x

        g5 = (char) (Math.floor(((-lon + 180) % 2) / (2.0 / 24.0)) + 97);
        g6 = (char) ((Math.floor(((lat + 90) % 1) / (1.0 / 24.0))) + 97);
        str = str + g1 + g2 + g3 + g4 + g5 + g6;

        return str;

    }
    /* Returns Point2D.Double Lat/Lon pair for given X/Y pixel */

    private java.awt.geom.Point2D.Double getLatLon(int x, int y) {
        double lat;
        double lon;
        java.awt.geom.Point2D.Double latlon = new java.awt.geom.Point2D.Double();

        latlon.x = refDegreesLat - degreesPerPixelY * y;
        latlon.y = refDegreesLon - degreesPerPixelX * x;

        return latlon;
    }

    /* Returns the 6-digit grid String for given X/Y pixel */
    private String getGrid(int x, int y) {
        String str = "";

        double lat;
        double lon;

        lat = refDegreesLat - degreesPerPixelY * y;
        lon = refDegreesLon - degreesPerPixelX * x;
        str = getGrid(lat, lon);
        Point p = getPixelFromGrid(str);
        return str;
    }

    /*returns Point2D.Double Lat/Lon pair for center of given grid string */
    private java.awt.geom.Point2D.Double getLatLon(String grid) {
        double lat;
        double lon;
        java.awt.geom.Point2D.Double latlon = new java.awt.geom.Point2D.Double();
        char[] gridArray = grid.toCharArray();

        lon = (180 - ((gridArray[0] - 65) * 20.0))
                - ((gridArray[2] - 48) * 2.0)
                - ((gridArray[4] - 97) * (2.0 / 24.0))
                - 1.0 / 24.0;  //center of square
        lat = (((gridArray[1] - 65) * 10.0) - 90)
                + (gridArray[3] - 48)
                + ((gridArray[5] - 97) * (1.0 / 24.0))
                + 0.5 / 24.0;  //center of square

        latlon.x = lat;
        latlon.y = lon;
        return latlon;
    }

    /** Gets the local pixel for the center of a 6-digit grid.  String
     * should be checked by isGrid() first */
    public Point getPixelFromGrid(String str) {
        Point p = new Point();
        double lat;
        double lon;
        double distance;
        double direction;

        java.awt.geom.Point2D.Double latlon = getLatLon(str);
        lat = latlon.x;
        lon = latlon.y;

        p.y = (int) ((refDegreesLat - lat) / degreesPerPixelY);
        p.x = (int) ((refDegreesLon - lon) / degreesPerPixelX);

        return p;
    }
    /* returns distance in miles from arc in radians (assumes spherical earth) */

    private double radiansToMiles(double rad) {
        double miles;
        miles = (rad * 180 * 60 / Math.PI) * 1.15077945;  //to statute miles
        return miles;
    }

    /*
     *Height is in 1000's of feet.  Returns true if storm is above horizon for the station location
     */
    private boolean isVisible(java.awt.geom.Point2D.Double stationLocation, java.awt.geom.Point2D.Double stormLocation, double ht) {
        boolean visible = false;
        double height = ht * 1000;
        double stationLat = stationLocation.x;
        double stationLon = stationLocation.y;
        double stormLat = stormLocation.x;
        double stormLon = stormLocation.y;
        double distance = radiansToMiles(getDistance(stationLat, stationLon, stormLat, stormLon));
        double radius = getStormRadiusInMiles(height);
        if (distance < radius) {
            visible = true;
        }
        return visible;
    }

    /**
     * Compares the list of stations in StationListWindow with the storm table
     * to find which paths might exist between stations.  Public b/cIt is called by
     * StormTableReader after downloading the storm table.
     */
    public void findPaths() {
        oldDXGrid = dxGridField.getText();
        String alertText = "";  //text for the alert
        getVisibleStationsButtons();
        visibleStationsPanel.removeAll(); //clear all old buttons
        stormTable = new java.util.Vector(stormTableReader.getStormTable());
        stationList = new java.util.Vector(stationListWindow.getStations());
        myStormTable = new java.util.Vector();  //storms visible at my location
        dxStormTable = new java.util.Vector();  //storms visible to both home and dx
        java.util.Vector[] otherStationStormTables = new java.util.Vector[stationList.size()];
        for (int i = 0; i < otherStationStormTables.length; i++) {
            otherStationStormTables[i] = new java.util.Vector();
        }
        for (int i = 0; i < stormTable.size(); i++) {   //for each storm in Table
            Storm storm = new Storm();
            storm = (Storm) stormTable.elementAt(i);
            java.awt.geom.Point2D.Double stormLatLon = new java.awt.geom.Point2D.Double(storm.getLat(), storm.getLon());
            java.awt.geom.Point2D.Double dxLatLon = getLatLon(oldDXGrid);
            double height = storm.getTop();
            boolean me = isVisible(homeLocation, stormLatLon, height);  //storm visible from my location
            // boolean dx = isVisible(dxLocation, stormLatLon, height);   //storm visible from dxLocation
            boolean dx = isVisible(dxLatLon, stormLatLon, height);   //storm visible from dxLocation
            if (me) {
                myStormTable.add(storm);
                if (dx) {
                    dxStormTable.add(storm);
                }
            }
        }

        //do the station list now
        for (int i = 0; i < otherStationStormTables.length; i++) {
            Station s = new Station();
            s = (Station) stationList.elementAt(i);
            java.awt.geom.Point2D.Double stationLatLon = getLatLon(s.getGrid());
            for (int j = 0; j < stormTable.size(); j++) {
                Storm storm = new Storm();
                storm = (Storm) stormTable.elementAt(j);
                java.awt.geom.Point2D.Double stormLatLon = new java.awt.geom.Point2D.Double(storm.getLat(), storm.getLon());
                double height = storm.getTop();
                boolean other = isVisible(stationLatLon, stormLatLon, height); //storm visible from station in list
                boolean me = isVisible(homeLocation, stormLatLon, height);
                if (other && me) {
                    otherStationStormTables[i].add(storm);
                }
            }
        }
        int numberOfStations = stationList.size();
        visibleStationsVector = new java.util.Vector();  //every station that is mutually visible
        visibleStationsStormTables = new java.util.Vector(); //corresponding tables



        for (int i = 0; i < numberOfStations; i++) {
            if (otherStationStormTables[i].size() > 0) {  //if storms present in the storm table
                Station s = new Station();                //add them to the window
                s = (Station) stationList.elementAt(i);
                String call = s.getCall();
                visibleStationsVector.add(call);
                visibleStationsStormTables.add(otherStationStormTables[i]);
                visibleStationsPanel.add(visibleStationsButtons[i]);
            }

        }
        if (dxStormTable.size() > 0) {     //if storms present add DX button to window
            visibleStationsVector.add(" DX ");
            visibleStationsPanel.add(visibleStationsButtons[visibleStationsButtons.length - 1]);
        }
        visibleStationsPanel.revalidate();

        if (alarmSettingsWindow.getAlarmType() == 1) {  //popup windows
            int response = -1;  //initial condx for response

            switch (alarmSettingsWindow.getAlarmCondition()) {

                case (1): {      //storms at my location
                    if (myStormTable.size() > 0) {
                        Object[] options = {"View Storms", "Close"};  //for Option pane below
                        response = JOptionPane.showOptionDialog(
                                this, "Storm visible from your location:", "Alert!", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, options, options[0]);
                        if (response == 0) {   //user selected "View Storms"
                            stormTableViewer.dispose();
                            stormTableViewer = new StormViewer(myStormTable);
                            stormTableViewer.setTitle("Storms visible from home location");
                            stormTableViewer.setVisible(true);
                        }
                    }
                    break;
                }
                case (2): {      //storms visible with DX or stations in list
                    //first check for DX location
                    if (dxStormTable.size() > 0) {
                        JOptionPane.showMessageDialog(this, "Possible path to DX location!", "Alert!", JOptionPane.INFORMATION_MESSAGE);
                    }
                    if (visibleStationsStormTables.size() > 0) {
                        JOptionPane.showMessageDialog(this, "Possible path to Station in List!", "Alert!", JOptionPane.INFORMATION_MESSAGE);
                    }

                    break;
                }
                case (3): {      //just stations in list
                    if (visibleStationsVector.size() > 0) {
                        JOptionPane.showMessageDialog(this, "Possible path to Station in List!", "Alert!", JOptionPane.INFORMATION_MESSAGE);
                    }
                    break;
                }
                case (4): {      //just dx location
                    if (dxStormTable.size() > 0) {
                        JOptionPane.showMessageDialog(this, "Possible path to DX location!", "Alert!", JOptionPane.INFORMATION_MESSAGE);
                    }
                }
            }
        }

        visibleStationsPanel.repaint();
        visibleStationsPanel.validate();
    }

    private void getVisibleStationsButtons() {
       
        java.util.Vector stationList = stationListWindow.getStations();
        
        for (JToggleButton b : visibleStationsButtons) {
            visibleStationsButtonGroup.remove(b);
        }
        visibleStationsButtonGroup = new ButtonGroup();
        visibleStationsButtons = new JToggleButton[stationList.size() + 1];
        
        for (int i = 0; i < stationList.size(); i++) {
            Station s = (Station) stationList.elementAt(i);
            visibleStationsButtons[i] = new JToggleButton(s.getCall());
            visibleStationsButtons[i].setToolTipText("Click to see mutually visible" + '\n' + "storms with this Location");
            visibleStationsButtons[i].addActionListener(this);
            visibleStationsButtonGroup.add(visibleStationsButtons[i]);
        }
   
        visibleStationsButtons[stationList.size()] = new JToggleButton(" DX ");
        visibleStationsButtons[stationList.size()].setToolTipText("Click to see mutually visible storms with DX location");
        visibleStationsButtons[stationList.size()].addActionListener(this);
        visibleStationsButtonGroup.add(visibleStationsButtons[stationList.size()]);
    }

    /*displays the entire storm table in separate window */
    private void viewStormTable() {
        stormTableViewer.dispose();   //get rid of old window if it is present
        stormTable = new java.util.Vector(stormTableReader.getStormTable());
        stormTableViewer = new StormViewer(stormTable);
        stormTableViewer.setVisible(true);
    }

    /** called by the Alarm Setting Window to update the alarm on main program
     * window when the alarm settings change
     */
    public void updateAlarmLabel() {
        if (autoCheckBox.isSelected()) {
            if (alarmSettingsWindow.getAlarmType() == alarmSettingsWindow.ALARM_ON) {  // if alarms are enabled
                alarmLabel.setText("ON");
            } else {
                alarmLabel.setText("OFF");
            }
        } else {
            alarmLabel.setText("OFF");
        }
    }

    /** Checks a string to see if it is a valid 6-digit square in the form AAnnaa */
    private boolean isGrid(String string) {
        boolean b = false;
        char[] grid = string.toCharArray();

        if (grid.length == 6) {
            if ((int) grid[0] > 64 && (int) grid[0] < 83) {
                if ((int) grid[1] > 64 && (int) grid[1] < 83) {
                    if ((int) grid[2] > 47 && (int) grid[2] < 58) {
                        if ((int) grid[3] > 47 && (int) grid[3] < 58) {
                            if ((int) grid[4] > 96 && (int) grid[4] < 121) {
                                if ((int) grid[5] > 96 && (int) grid[5] < 121) {
                                    b = true;
                                }
                            }
                        }
                    }
                }
            }
        }
        return b;
    }

    /** returns the radar-specific part of the URL for each radar string.  These
     * strings are selected by the radarSelectionComboBox. */
    private String getRadarURL(String str) {
        String line = "";
        String radar = "";
        try {
            File radardata = new File("radarlocations.txt");
            RandomAccessFile raf = new RandomAccessFile(radardata, "r");
            //find str in file

            boolean found = false;
            while (!found) {
                line = raf.readLine();

                java.util.StringTokenizer st = new java.util.StringTokenizer(line, "|");
                String[] data = new String[st.countTokens()];
                int count = st.countTokens();
                for (int i = 0; i < count; i++) {
                    data[i] = st.nextToken();
                }
                if (data[0].equals(str)) {
                    radar = data[1].substring(1, 4); //remove the prefix K/P/T etc. of ID
                    found = true;

                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        //System.out.println("RADAR = " + radar);
        return radar;
    }

    public boolean isMaxReflectivitySelected() {
        return heightCheckBoxMenuItem.isSelected();
    }

    public boolean isEchoTopSelected() {
        return echoTopsCheckBoxMenuItem.isSelected();
    }

    private void saveSettings() {


        node.put("myGrid", myGrid);
        node.put("dxGrid", dxGrid);
        node.put("currentRadarName", currentRadarName);
        node.put("currentRadarView", currentRadarView);
        node.putDouble("homeLocation.x", homeLocation.x);
        node.putDouble("homeLocation.y", homeLocation.y);
        node.putDouble("dxLocation.x", dxLocation.x);
        node.putDouble("dxLocation.y", dxLocation.y);
        node.putDouble("scatLocation.x", scatLocation.x);
        node.putDouble("scatLocation.y", scatLocation.y);


        /*now save checkboxes */
        node.putBoolean("topoCheckBox", topoCheckBox.isSelected());
        node.putBoolean("countiesCheckBox", countiesCheckBox.isSelected());
        node.putBoolean("citiesCheckBox", countiesCheckBox.isSelected());
        node.putBoolean("highwaysCheckBox", highwaysCheckBox.isSelected());
        node.putBoolean("riversCheckBox", riversCheckBox.isSelected());
        node.putBoolean("warningsCheckBox", warningsCheckBox.isSelected());
        node.putBoolean("legendCheckBox", legendCheckBox.isSelected());
        node.putBoolean("stationsCheckBox", stationsCheckBox.isSelected());

        node.putBoolean("echoTopsCheckBoxMenuItem", echoTopsCheckBoxMenuItem.isSelected());
        node.putBoolean("heightCheckBoxMenuItem", heightCheckBoxMenuItem.isSelected());
        node.putBoolean("jumpCheckBoxMenuItem", jumpCheckBoxMenuItem.isSelected());

        node.putBoolean("loadMapCheckBox", loadMapCheckBox.isSelected());
        node.putBoolean("loadStormsCheckBox", loadStormsCheckBox.isSelected());

        node.putInt("updateInterval", updateInterval);
    }

    private void loadSettings() {


        degreesPerPixelX = node.getDouble("degreesPerPixelX", 0.00982652143998578); //default for Buffalo NY
        degreesPerPixelY = node.getDouble("degreesPerPixelY", 0.00982652143998578); //shortrange base
        refDegreesLat = node.getDouble("refDegreesLat", 45.646381447532);
        refDegreesLon = node.getDouble("refDegreesLon", 81.6800426829945);

        refLatLon = new java.awt.geom.Point2D.Double(refDegreesLat, refDegreesLon);


        myGrid = node.get("myGrid", "FN13ed");
        dxGrid = node.get("dxGrid", "FN13ed");
        currentRadarName = node.get("currentRadarName", "Buffalo, NY");
        currentRadar = getRadarURL(currentRadarName);

        currentRadarView = node.get("currentRadarView", "N0R");

        homeLocation.x = node.getDouble("homeLocation.x", 0.0);
        homeLocation.y = node.getDouble("homeLocation.y", 0.0);
        dxLocation.x = node.getDouble("dxLocation.x", 0.0);
        dxLocation.y = node.getDouble("dxLocation.y", 0.0);
        scatLocation.x = node.getDouble("scatLocation.x", 0.0);
        scatLocation.y = node.getDouble("scatLocation.y", 0.0);



    }

    private void loadCheckBoxSettings() {
        topoCheckBox.setSelected(node.getBoolean("topoCheckBox", true));
        countiesCheckBox.setSelected(node.getBoolean("countiesCheckBox", true));
        citiesCheckBox.setSelected(node.getBoolean("citiesCheckBox", true));
        riversCheckBox.setSelected(node.getBoolean("riversCheckBox", true));
        highwaysCheckBox.setSelected(node.getBoolean("highwaysCheckBox", true));
        warningsCheckBox.setSelected(node.getBoolean("warningsCheckBox", true));
        stationsCheckBox.setSelected(node.getBoolean("stationsCheckBox", true));
        legendCheckBox.setSelected(node.getBoolean("legendCheckBox", true));

        echoTopsCheckBoxMenuItem.setSelected(node.getBoolean("echoTopsCheckBoxMenuItem", false));
        heightCheckBoxMenuItem.setSelected(node.getBoolean("heightCheckBoxMenuItem", false));

        jumpCheckBoxMenuItem.setSelected(node.getBoolean("jumpCheckBoxMenuItem", false));

        loadMapCheckBox.setSelected(node.getBoolean("loadMapCheckBox", false));
        loadStormsCheckBox.setSelected(node.getBoolean("loadStormsCheckBox", false));

        autoCheckBox.setSelected(node.getBoolean("autoCheckBox", false));
        updateInterval = node.getInt("updateInterval", 5);
        //jSpinner1.setValue(new Integer(updateInterval));

    }

    private void initRadarButtons() {

        if (currentRadarView.equals(shortBaseRadar)) {
            //shortrangeButton.setSelected(true);
            baseButton.setSelected(true);
            // milesPerPixel = SHORTBASE;
        } else if (currentRadarView.equals(shortCompRadar)) {

            compositeButton.setSelected(true);

        } else if (currentRadarView.equals(shortTopRadar)) {

            this.echoTopsButton.setSelected(true);

        } else if (currentRadarView.equals(shortVILRadar)) {

            //vilButton.setSelected(true);

        } else if (currentRadarView.equals(longBaseRadar)) {
            longrangeButton.setSelected(true);

        }

    }

    public void saveRadarList() {


        ComboBoxModel m = radarSelectionComboBox.getModel();
        String str;
        int i = 0;
        for (i = 0; i < m.getSize(); i++) {
            str = (String) m.getElementAt(i);
            switch (i) {
                case (0): {
                    node.put("radarList0", str);
                    break;
                }
                case (1): {
                    node.put("radarList1", str);
                    break;
                }
                case (2): {
                    node.put("radarList2", str);
                    break;
                }
                case (3): {
                    node.put("radarList3", str);
                    break;
                }
                case (4): {
                    node.put("radarList4", str);
                    break;
                }
                case (5): {
                    node.put("radarList5", str);
                    break;
                }
                case (6): {
                    node.put("radarList6", str);
                    break;
                }
                case (7): {
                    node.put("radarList7", str);
                    break;
                }
                case (8): {
                    node.put("radarList8", str);
                    break;
                }
                case (9): {
                    node.put("radarList9", str);
                    break;
                }

            }
        }
        while (i < 10) {
            switch (i) {
                case (0): {
                    node.remove("radarList0");
                    i++;
                    break;
                }
                case (1): {
                    node.remove("radarList1");
                    i++;
                    break;
                }
                case (2): {
                    node.remove("radarList2");
                    i++;
                    break;
                }
                case (3): {
                    node.remove("radarList3");
                    i++;
                    break;
                }
                case (4): {
                    node.remove("radarList4");
                    i++;
                    break;
                }
                case (5): {
                    node.remove("radarList5");
                    i++;
                    break;
                }
                case (6): {
                    node.remove("radarList6");
                    i++;
                    break;
                }
                case (7): {
                    node.remove("radarList7");
                    i++;
                    break;
                }
                case (8): {
                    node.remove("radarList8");
                    i++;
                    break;
                }
                case (9): {
                    node.remove("radarList9");
                    i++;
                    break;
                }

            }

        }
    }

    private void loadRadarList() {

        String[] strs = new String[10];
        strs[0] = node.get("radarList0", null);
        strs[1] = node.get("radarList1", null);
        strs[2] = node.get("radarList2", null);
        strs[3] = node.get("radarList3", null);
        strs[4] = node.get("radarList4", null);
        strs[5] = node.get("radarList5", null);
        strs[6] = node.get("radarList6", null);
        strs[7] = node.get("radarList7", null);
        strs[8] = node.get("radarList8", null);
        strs[9] = node.get("radarList9", null);

        DefaultComboBoxModel model = new DefaultComboBoxModel();

        for (int i = 0; i < strs.length; i++) {
            if (strs[i] != null) {
                model.addElement(strs[i]);
            }
        }
        model.setSelectedItem(currentRadarName);
        radarSelectionComboBox.setModel(model);

    }

    /** Gets radius in miles for given storm height in feet*/
    private double getStormRadiusInMiles(double h) {
        double radius = 20912696.6 * Math.acos(20912696.6 / (20912696.6 + h)) / 5280;
        return radius;

    }

    /**
     * This will set the scattering location on the map to the location of the storm.
     * It will also set the dimensions of the storm circles if selected.
     * @param storm the Storm to be displayed
     */
    public void displayStormFromTable(Storm storm) {
        map.setStorm(storm);

        scatLocation.x = storm.getLat();
        scatLocation.y = storm.getLon();
        String stormRadar = storm.getRadar();
        stormRadar = stormRadar.substring(1, 4).toUpperCase();

        String grid = getGrid(scatLocation.x, scatLocation.y);
        Point p = getPixelFromGrid(grid);
        displayInfo(p.x, p.y);

        System.out.println(stormRadar + "  current Radar: " + currentRadar);
        if (currentRadar.equals(stormRadar)) {
            paintLocations();
        } else {
            //if check box is selected, load radar map for this storm
            if (jumpCheckBoxMenuItem.isSelected()) {
                String s = storm.getRadar();

                //str = str.substring(1,3).toUpperCase();
                if (!currentRadar.equals(s)) {  //move to that map if we aren't already there
                    currentRadar = s;
                    updateMap();
                }
            }
            paintLocations();
        }
    }

    private void setLargeGUI() {

        initComponents();
        postInit();
    }

    /**
     * This is the main exectution method.  Called when executing the program.
     * @param args arguements passed at execution (none used at this time)
     */
    public static void main(String args[]) {
        RainScatter rs = new RainScatter();
        rs.setVisible(true);
    }

    /**
     * For the visible station buttons and meunu checkboxes.  Other components have
     * their own private action handlers.
     * @param e the ActionEvent
     */
    public void actionPerformed(java.awt.event.ActionEvent e) {
        Object obj = e.getSource();

        if (obj instanceof JToggleButton && !(obj instanceof JCheckBox)) {   //if obj is a JButton
            JToggleButton b = (JToggleButton) (obj);
            if (b.getParent() == visibleStationsPanel) {  //button is a visible station button
                visibleStormsPanel.removeAll();
                String call = b.getText();
                if (call.equals(" DX ")) {  //DX button was clicked
                    //visibleStormsPane.removeAll(); //clear any previous storm table
                    StormViewerPane svp = new StormViewerPane(dxStormTable);
                    //visibleStormsPane.add(svp);
                    JTable table = svp.getTable();
                    
                    // System.out.println(table.getRowCount());
                    //visibleStormsPane.add(table);
                    JScrollPane p = new JScrollPane(table);
                    //table.setSize(visibleStormsPanel.getSize());
                    p.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                    p.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
                    dxGridField.setText(oldDXGrid);
                    dxGridFieldActionPerformed(new java.awt.event.ActionEvent(dxGridField, 0, null));                   
                    visibleStormsPanel.add(p);
                    visibleStormsPanel.repaint();
                    jPanel1.repaint();

                } else {                           //one of the stations was clicked
                    java.util.Vector stationList = stationListWindow.getStations();
                    int stormTablePosition = -1;
                    String grid = "";
                    Station s = new Station();
                    // s = (Station)stationList.elementAt(i);  //This is not right!
                    for (int j = 0; j < stationList.size(); j++) {  //find call in stationList
                        if (((Station) stationList.elementAt(j)).getCall().equals(call)) {
                            s = (Station) stationList.elementAt(j);
                        }
                    }
                    grid = s.getGrid();
                    for (int i = 0; i < visibleStationsVector.size(); i++) {
                        if (call.equals(visibleStationsVector.elementAt(i))) {
                            stormTablePosition = i;
                        }
                    }
                    //now set the dx location to the other station
                    dxGridField.setText(grid);
                    dxGridFieldActionPerformed(new java.awt.event.ActionEvent(dxGridField, 0, "noFindPaths"));

                    //javax.swing.border.TitledBorder title;
                    //title = BorderFactory.createTitledBorder(call);
                    //visibleStormsPanel.setBorder(title);
                   //visibleStormsPanel.removeAll(); //clear any previous storm table
                    StormViewerPane svp = new StormViewerPane((java.util.Vector) (visibleStationsStormTables).elementAt(stormTablePosition));
                    JTable table = svp.getTable();
                    
                    // System.out.println(table.getRowCount());
                    //visibleStormsPane.add(table);
                    JScrollPane p = new JScrollPane(table);
                     p.setSize(visibleStormsPanel.getSize());
                    p.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
                               
                    visibleStormsPanel.add(p, BorderLayout.CENTER);
                    visibleStormsPanel.repaint();
                    jPanel1.repaint();
                    //visibleStormsPanel.validate();
                    
                }
                visibleStormsPanel.revalidate();
                jPanel1.revalidate();
            }
        } else if (obj instanceof javax.swing.JCheckBoxMenuItem) {

            JCheckBoxMenuItem checkBox = (JCheckBoxMenuItem) obj;
            paintLocations();


        } 

    }

    ////////////////////                                    ////////////////////
    ///////////////////  Other clases used by RainScatter  ///////////////////
    //////////////////                                    ///////////////////
    class MyTableModel extends javax.swing.table.AbstractTableModel
            implements javax.swing.event.TableModelListener {

        java.util.Vector stormTableVector;
        java.util.Vector columnNames;

        public MyTableModel(java.util.Vector stormTableVector, java.util.Vector columnNames) {
        }

        public boolean isCellEditable(int row, int col) {
            return false;
        }

        public int getColumnCount() {
            if (stormTableVector.size() > 0) {
                Object obj = stormTableVector.elementAt(0);
                java.util.Vector vect = (java.util.Vector) obj;
                return vect.size();
            } else {
                return 0;
            }
        }

        public int getRowCount() {
            return stormTableVector.size();
        }

        public Object getValueAt(int row, int col) {
            Object obj = stormTableVector.elementAt(row);
            java.util.Vector vect = (java.util.Vector) obj;
            return vect.elementAt(col);
        }

        public String getColumnName(int col) {
            columnNames = getColumnHeader();
            return (String) columnNames.elementAt(col);
        }

        public void tableChanged(javax.swing.event.TableModelEvent e) {
        }

        private java.util.Vector getColumnHeader() {
            java.util.Vector vect = new java.util.Vector();
            vect.add("Grid");
            vect.add("Top");
            vect.add("dBz");
            vect.add("Height");
            vect.add("Dir");
            vect.add("Speed");
            vect.add("ID");
            vect.add("TVS");
            vect.add("MSO");
            vect.add("POSH");
            vect.add("POH");
            vect.add("MXSize");
            vect.add("VIL");
            vect.add("Rad");
            return vect;
        }
    }

    ////////                         ////////
    ////////  Radar selection frame  ////////
    /**
     * This class creates a frame for the Radar List and populates it with all known
     * NEXRAD Radar locations.  The user can then select from this list to set the
     * radarsList in RainScatter.  This is the list that appears in the
     * radarSelectionComboBox as well as that of the storm table.
     */
    public class RadarListFrame extends javax.swing.JFrame {

        int numRadars = 0;
        private javax.swing.JButton cancelButton;
        private javax.swing.JPanel jPanel1;
        private javax.swing.JPanel jPanel2;
        private javax.swing.JButton okButton;
        private javax.swing.JCheckBox[] boxes;
        private javax.swing.JScrollPane scrollPane;

        /**
         * Creates radar list.
         * @param radarsObj the String[] of radars.  Must be of type String[] or throws exception
         */
        public RadarListFrame(Object radarsObj) {
            //setLocationRelativeTo(null);
            setDefaultCloseOperation(DISPOSE_ON_CLOSE);
            initComponents();

            String[] radars = (String[]) radarsObj;
            numRadars = radars.length;
            boxes = new javax.swing.JCheckBox[numRadars];
            for (int i = 0; i < radars.length; i++) {
                boxes[i] = new javax.swing.JCheckBox();
                boxes[i].setText(radars[i]);
                jPanel1.add(boxes[i], i);
                //scrollPane.add(boxes[i]);
            }
            setCheckBoxes();
            jPanel1.validate();

            pack();
            setResizable(true);
            setTitle("Select 1-10 radars for pulldown menu");

        }

        /** This method is called from within the constructor to
         * initialize the form.
         */
        private void initComponents() {
            jPanel1 = new javax.swing.JPanel();
            jPanel2 = new javax.swing.JPanel();
            okButton = new javax.swing.JButton();
            cancelButton = new javax.swing.JButton();

            scrollPane = new javax.swing.JScrollPane(jPanel1);

            //    getContentPane().setLayout(new javax.swing.BoxLayout(getContentPane(), javax.swing.BoxLayout.Y_AXIS));
            getContentPane().setLayout(new java.awt.BorderLayout());

            addWindowListener(new java.awt.event.WindowAdapter() {

                public void windowClosing(java.awt.event.WindowEvent evt) {
                    exitForm(evt);
                }
            });

            jPanel1.setLayout(new java.awt.GridLayout(numRadars, 1));
            //scrollPane.setLayout(1, numRadars);


            getContentPane().add(scrollPane, java.awt.BorderLayout.CENTER);


            okButton.setText("OK");
            okButton.addMouseListener(new java.awt.event.MouseAdapter() {

                public void mouseClicked(java.awt.event.MouseEvent evt) {
                    okButtonMouseClicked(evt);
                }
            });

            jPanel2.add(okButton);

            cancelButton.setText("Cancel");
            cancelButton.addMouseListener(new java.awt.event.MouseAdapter() {

                public void mouseClicked(java.awt.event.MouseEvent evt) {
                    cancelButtonMouseClicked(evt);
                }
            });

            jPanel2.add(cancelButton);


            getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);
            setSize(300, 600);
            pack();
        }

        private void setCheckBoxes() {

            ComboBoxModel m = radarSelectionComboBox.getModel();
            int len = m.getSize();
            for (int i = 0; i < len; i++) {            //for each element in model
                String string = (String) m.getElementAt(i);
                for (int j = 0; j < numRadars; j++) {       //for each checkbox
                    if (string.equals(boxes[j].getText())) {
                        boxes[j].setSelected(true);
                    }
                }
            }
        }

        private void okButtonMouseClicked(java.awt.event.MouseEvent evt) {

            int j = 0;
            for (int i = 0; i < numRadars; i++) {
                if (boxes[i].isSelected()) {
                    j++;
                }
            }
            String[] strs = new String[j];
            j = 0;
            for (int i = 0; i < numRadars; i++) {
                if (boxes[i].isSelected()) {
                    strs[j] = boxes[i].getText();
                    j++;
                }
            }
            if (strs.length == 0) {
                javax.swing.JOptionPane.showMessageDialog(this, "You must select at least one radar.");
            } else if (strs.length > 10) {
                javax.swing.JOptionPane.showMessageDialog(this, "Please select no more than 10 radar sites at once.");
            } else {
                changeRadarList(strs);
                saveRadarList();
                dispose();
            }
        }

        private void cancelButtonMouseClicked(java.awt.event.MouseEvent evt) {
            dispose();
        }

        /** close without changing anything*/
        private void exitForm(java.awt.event.WindowEvent evt) {
            dispose();
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem AlarmSettingsMenuItem;
    private javax.swing.JMenuItem aboutMenuItem;
    public javax.swing.JLabel alarmLabel;
    private javax.swing.JCheckBox autoCheckBox;
    private javax.swing.JRadioButton baseButton;
    private javax.swing.JPanel bearingPanel;
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JCheckBox citiesCheckBox;
    private javax.swing.JLabel cityMap;
    private javax.swing.JRadioButton compositeButton;
    private javax.swing.JCheckBox countiesCheckBox;
    private javax.swing.JLabel countyMap;
    private javax.swing.JMenu drawMenu;
    private javax.swing.JTextField dxGridField;
    private javax.swing.JRadioButton echoTopsButton;
    private javax.swing.JCheckBoxMenuItem echoTopsCheckBoxMenuItem;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.ButtonGroup fishEyeButtonGroup;
    private javax.swing.JMenu footprintMenu;
    private javax.swing.JLabel gridLabel;
    private javax.swing.ButtonGroup guiButtonGroup;
    private javax.swing.JCheckBoxMenuItem heightCheckBoxMenuItem;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JLabel highwayMap;
    private javax.swing.JCheckBox highwaysCheckBox;
    private javax.swing.JTextField homeGridField;
    private javax.swing.JProgressBar imageProgressBar;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JLayeredPane jLayeredPane1;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JSpinner jSpinner1;
    private javax.swing.JTextField jTextField3;
    private javax.swing.JTextField jTextField4;
    private javax.swing.JTextField jTextField8;
    private javax.swing.JTextField jTextField9;
    private javax.swing.JCheckBoxMenuItem jumpCheckBoxMenuItem;
    private javax.swing.JCheckBox legendCheckBox;
    private javax.swing.JLabel legendMap;
    private javax.swing.JCheckBoxMenuItem loadMapCheckBox;
    private javax.swing.JCheckBoxMenuItem loadStormsCheckBox;
    private javax.swing.JRadioButton longrangeButton;
    private javax.swing.JPanel mapPanel;
    private javax.swing.JPanel myStationPanel;
    private javax.swing.JPanel otherStationPanel;
    private javax.swing.JPanel pathPanel;
    private javax.swing.JMenuItem quickStartMenuItem;
    private javax.swing.ButtonGroup radarButtonGroup1;
    private javax.swing.JMenuItem radarListMenuItem;
    private javax.swing.JPanel radarOverlayPanel;
    private javax.swing.JPanel radarPanel;
    private javax.swing.JComboBox radarSelectionComboBox;
    private javax.swing.ButtonGroup radarSelectionGroup;
    private javax.swing.JPanel radarSelectionPanel;
    private javax.swing.JPanel radarUpdatePanel;
    private javax.swing.JPanel radarViewPanel;
    private javax.swing.JLabel riverMap;
    private javax.swing.JCheckBox riversCheckBox;
    private javax.swing.JToggleButton setDXButton;
    private javax.swing.JToggleButton setHomeButton;
    private javax.swing.JMenu settingsMenu;
    private javax.swing.JMenu startupMenu;
    private javax.swing.JMenuItem stationListMenuItem;
    private javax.swing.JCheckBox stationsCheckBox;
    public javax.swing.JLabel statusLabel;
    public javax.swing.JProgressBar stormTableProgressBar;
    private javax.swing.JCheckBox topoCheckBox;
    private javax.swing.JLabel topoMap;
    private javax.swing.JButton updateMapButton;
    private javax.swing.JButton updateStormsButton;
    private javax.swing.JButton viewStormsButton;
    private javax.swing.ButtonGroup visibleStationsButtonGroup;
    private javax.swing.JPanel visibleStationsPanel;
    private javax.swing.JPanel visibleStormsPanel;
    private javax.swing.JLabel warningMap;
    private javax.swing.JCheckBox warningsCheckBox;
    // End of variables declaration//GEN-END:variables
}
