/*
 * StationListWindow.java
 *
 * Created on January 7, 2005, 3:19 PM
 */

/**
 *
 * @author  Andrew Flowers, K0SM
 */


import java.util.*;
import javax.swing.*;
import java.io.*;

/** This class handles the list of stations and respective grid squares for
 * checking agains the storm table.  The list of stations is stored to disk in 
 * stations.dat.
 */

public class StationListWindow extends javax.swing.JFrame {
    //this is used to refer back to the main program
    final RainScatter MAIN_PROGRAM =                   //this is used to refer
    (RainScatter)RainScatter.getFrames()[0];   //to the main program
    javax.swing.JTextField[] stationList;
    Vector stations = new Vector();
    
    /** Creates new form StationListWindow */
    public StationListWindow() {
        initComponents();
        populateStationList();
        readStationList();
        setSize(270, 420);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jScrollPane1 = new javax.swing.JScrollPane();
        stationPanel = new javax.swing.JPanel();
        okayButton = new javax.swing.JButton();

        setTitle("Station List");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        stationPanel.setLayout(new java.awt.GridLayout(60, 2));

        jScrollPane1.setViewportView(stationPanel);

        getContentPane().add(jScrollPane1, java.awt.BorderLayout.CENTER);

        okayButton.setText("OK");
        okayButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okayButtonActionPerformed(evt);
            }
        });

        getContentPane().add(okayButton, java.awt.BorderLayout.SOUTH);

        pack();
    }//GEN-END:initComponents
    
    private void okayButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okayButtonActionPerformed
        boolean b = saveStations();  //b not used, just necessary for call
        this.dispose();
    }//GEN-LAST:event_okayButtonActionPerformed
    /*returns true if data in list is valid, return false if it is improperly formatted */
    private boolean saveStations() {
        
        //check to make sure callsigns contain no spaces
        boolean okayData = true;
        for (int i = 0; i < stationList.length; i++) {
            String s = stationList[i].getText();
            
            for (int j = 0; j < s.length(); j++) {
                if (s.charAt(j) == ' ') {
                    javax.swing.JOptionPane.showMessageDialog(this,
                    "One or more of the callsigns contains a space." + '\n' + "Please fix before saving",
                    "Fix callsign format", javax.swing.JOptionPane.WARNING_MESSAGE);
                    okayData = false;
                    break;
                }
            }
            i++; //increment one to get to grid field
            String grid = stationList[i].getText();
            grid = trimString(grid); //remove any spaces in grid
            grid = fixGrid(grid);
            stationList[i].setText(grid);
            if (!isGrid(grid) && !grid.equals("")) {  //textfields can be null too
                javax.swing.JOptionPane.showMessageDialog(this,
                "Gridsquare for " + stationList[i-1].getText() + " is not a legal 6-digit grid (AAnnaa format)" + '\n' + "Please fix before saving",
                "Fix grid format", javax.swing.JOptionPane.WARNING_MESSAGE);
                okayData = false;
                break;
            }
        }
        if (okayData) {  //all textfields pass the test
            File file = new File("stations.dat");
            try {
                FileWriter out = new FileWriter(file);
                
                for (int i = 0; i < stationList.length; i++) {
                    out.write(stationList[i].getText() + " " + stationList[i+1].getText() + '\n');
                    i++;
                }
                out.close();
            }
            catch (IOException e) {e.printStackTrace();}
            readStationList();
            setVisible(false);  //hide the window
        }
        return okayData;
    }
    
    private boolean isGrid(String str) {
        boolean b = false;
        char[] grid = str.toCharArray();
        
        if (grid.length == 6) {
            if ( (int)grid[0] > 64 && (int)grid[0] < 83) {
                if ( (int)grid[1] > 64 && (int)grid[1] < 83) {
                    if ( (int)grid[2] > 47 && (int)grid[2] < 58) {
                        if ( (int)grid[3] > 47 && (int)grid[3] < 58) {
                            if ( (int)grid[4] > 96 && (int)grid[4] < 121) {
                                if ( (int)grid[5] > 96 && (int)grid[5] < 121) {
                                    b = true;
                                }
                            }
                        }
                    }
                }
            }
        }
        return b;
    }
    private String fixGrid(String str) {
        
        //correct to AAnnaa format
        if (str.length() == 6) {
            String strBegin  = str.substring(0, 2).toUpperCase();
            String strMiddle = str.substring(2, 4);
            String strEnd    = str.substring(4, 6).toLowerCase();
            str = strBegin + strMiddle + strEnd;
        }
        return str;
    }
    
    /** cuts extra spaces and characters from a string */
    private String trimString(String str) {
        String s = "";
        for (int i = 0; i < str.length(); i++) {
            char c = str.charAt(i);
            if (c != ' '  && c != '<' && c != '>') {
                s = s + c;
            }
        }
        return s;
    }
    
    /** creates the textboxes in the GUI for the Station List */
    private void populateStationList() {
        stationList = new javax.swing.JTextField[118];
        javax.swing.JLabel callLabel = new javax.swing.JLabel("Call");
        javax.swing.JLabel gridLabel = new javax.swing.JLabel("Grid");
        stationPanel.add(callLabel);
        stationPanel.add(gridLabel);
        for (int i = 0; i < stationList.length; i++) {
            stationList[i] = new javax.swing.JTextField();
            stationPanel.add(stationList[i]);
        }
    }
    /**
     * Accessor used to get the current list of stations
     * @return A vector containing the Stations and grids.
     */    
    public Vector getStations() {
        return stations;
    }
    /** User closes window, hide w/o saving changes.*/
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        setVisible(false);
        readStationList(); //if any data was changed in the GUI it is lost
                            // and reinitalized for the next time the user
                            // views the window.
    }//GEN-LAST:event_exitForm
    
    /** Accessor to get the list of stations.  Returns a Vector of stations
     *in the form call1, grid1, call2, grid2, call3, grid3.....
     *
     public Vector getStations() {
        readStationList();
        return stations;
    }
    
    /**reads the Station List from the file stations.dat */
    private void readStationList() {
        stations = new java.util.Vector();
        try {
            FileReader fr = new FileReader("stations.dat");
            int i = 0;
            
            int eof = 0;
            while  (eof != -1) {
                String str = "";
                char c =0;
                while ( c != '\n') {
                    eof = fr.read();
                    c = (char)eof;
                    if (c != '\n') {
                        str = str + c;
                    }
                }
                StringTokenizer st = new StringTokenizer(str);
                String call;
                String grid;
                try {
                    call = (String)st.nextElement();
                    stationList[i].setText(call);
                    grid = (String)st.nextElement();
                    stationList[i+1].setText(grid);
                    Station stat = new Station(call, grid);
                    stations.add(stat);
                    i = i + 2;
                }
                catch (NoSuchElementException nse) {break;} //ignore--empty line, break loop
                
            }
            
        }
        catch(Exception e) {
            
            MAIN_PROGRAM.setStatusLabel("Error reading station list.", java.awt.Color.RED);
            
        }
    }


    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton okayButton;
    private javax.swing.JPanel stationPanel;
    // End of variables declaration//GEN-END:variables
    
}
