/*
 * StormViewer.java
 *
 * Created on December 25, 2004, 11:17 AM
 */

/**
 *
 * @author  Andrew T. Flowers, K0SM
 */


import javax.swing.event.*;
import javax.swing.table.TableModel;
import javax.swing.table.*;
import javax.swing.*;
import java.awt.event.MouseEvent;

/**
 * StormViewer is a Frame that contains a table displaying a storm table.
 */
public class StormViewer extends javax.swing.JFrame {
    
    protected String[] columnToolTips = {"Location of storm",
    "Height of Storm top (1000's ft)",
    "Max reflectivity of storm",
    "Height of max reflectivity",
    "Direction of movement (deg)",
    "Speed of movement(knots)",
    "Storm ID",
    "Tornadic Vortex Signature",
    "Mesocyclone Signature",
    "Possiblity of severe hail",
    "Possibility of hail",
    "Predicted max size of hail (in)",
    "Vertically integrated liquid water (kg/m^2)",
    "Reporting radar"};
    
    /** Creates empty StormViewer */
    public StormViewer() {
        
        initComponents();
        setSize(800,500);
        
    }
    
    /**
     * Creates a StormViewer for the particular stormTable data
     * @param stormTable Vector of Storms to be displayed in this StormViewer frame
     */
    public StormViewer(final java.util.Vector stormTable) {
        
        initComponents();
        
        
        stormTableVector = new java.util.Vector();
        
        int numberOfStorms = stormTable.size();
        Object[][] RowData;

        for (int i = 0; i < stormTable.size(); i++) {
            Storm storm = (Storm)stormTable.elementAt(i);
            java.util.Vector lineData = addStorm(storm);
            stormTableVector.add(lineData);
        }
        
        JTable table = new JTable(new MyTableModel()) {
            
            //Implement table header tool tips.
            protected JTableHeader createDefaultTableHeader() {
                return new JTableHeader(columnModel) {
                    public String getToolTipText(MouseEvent e) {
                        String tip = null;
                        java.awt.Point p = e.getPoint();
                        int index = columnModel.getColumnIndexAtX(p.x);
                        int realIndex = columnModel.getColumn(index).getModelIndex();
                        return columnToolTips[realIndex];
                    }
                };
            }
            
        };
        
        TableColumn column = null;
        for (int i = 0; i < table.getColumnCount(); i++) {
            column = table.getColumnModel().getColumn(i);
            switch (i) {
                    case 0: {column.setPreferredWidth(60); break;}
                    case 1: {column.setPreferredWidth(50); break;}
                    case 2: {column.setPreferredWidth(50); break;}
                    case 3: {column.setPreferredWidth(50); break;}
                    case 4: {column.setPreferredWidth(50); break;}
                    case 5: {column.setPreferredWidth(50); break;}
                    case 6: {column.setPreferredWidth(50); break;}
                    case 7: {column.setPreferredWidth(50); break;}
                    case 8: {column.setPreferredWidth(50); break;}
                    case 9: {column.setPreferredWidth(50); break;}
                    case 10: {column.setPreferredWidth(50); break;}
                    case 11: {column.setPreferredWidth(50); break;}
                    case 12: {column.setPreferredWidth(50); break;}
                    case 13: {column.setPreferredWidth(50); break;}
            }
        }
        //add table to the scrollpane
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        javax.swing.JScrollPane jsp = new javax.swing.JScrollPane(table);
        table.setPreferredScrollableViewportSize(new java.awt.Dimension(500,70));
        this.getContentPane().add(jsp);
        
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setToolTipText("Click on any row to set map to that storm");
        ListSelectionModel rowSM = table.getSelectionModel();
        rowSM.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                //Ignore extra messages.
                if (e.getValueIsAdjusting()) return;
                
                ListSelectionModel lsm = (ListSelectionModel)e.getSource();
                if (lsm.isSelectionEmpty()) {
                    
                }
                else {
                    int selectedRow = lsm.getMinSelectionIndex();
                    Storm s = (Storm)stormTable.elementAt(selectedRow);
                    
                    //This used to call back to the main program window for displaying the storm
                    java.awt.Frame[] mainProgramInstances = RainScatter.getFrames();
                    RainScatter rs = (RainScatter)mainProgramInstances[0];
                    rs.displayStormFromTable(s);
                }
            }
        });
        
        setSize(720, 500);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Alert!");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        pack();
    }//GEN-END:initComponents
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        this.dispose();
    }//GEN-LAST:event_exitForm
    /**
     * Sets the storms to be displayed
     * @param stormTable Vector of Storms to be displayed in this StormViewer
     */    
    public void setStormTable(java.util.Vector stormTable) {
        // initComponents();
        
        java.util.Vector stormTableVector = new java.util.Vector();
        
        int numberOfStorms = stormTable.size();
        Object[][] RowData;
        
        
        
        for (int i = 0; i < stormTable.size(); i++) {
            Storm storm = (Storm)stormTable.elementAt(i);
            java.util.Vector lineData = addStorm(storm);
            stormTableVector.add(lineData);
            
        }
       /*
        Object[][] rowData = new Object[numberOfStorms];
        for (int i =0; i < numberOfStorms; i++) {
            rowData[i] = stormTableVector.elementAt(i);
        }
        */
        table = new javax.swing.JTable(stormTableVector, getColumnHeader());
        javax.swing.JScrollPane jsp = new javax.swing.JScrollPane(table);
        table.setPreferredScrollableViewportSize(new java.awt.Dimension(500,70));
        /*
        if (table.getColumnCount() == 14) {
            for (int i = 0; i < 14; i++) {
                DefaultTableCellRenderer headerRenderer = new DefaultTableCellRenderer();
                table.getTab
                switch (i) {
                    case 0: {headerRenderer.setToolTipText("Location of storm"); break;}
                    case 1: {headerRenderer.setToolTipText("Height of Storm top (1000's ft)"); break;}
                    case 2: {headerRenderer.setToolTipText("Max reflectivity of storm"); break;}
                    case 3: {headerRenderer.setToolTipText("Height of max reflectivity"); break;}
                    case 4: {headerRenderer.setToolTipText("Direction of movement (deg)"); break;}
                    case 5: {headerRenderer.setToolTipText("Speed of movement(knots)"); break;}
                    case 6: {headerRenderer.setToolTipText("Storm ID"); break;}
                    case 7: {headerRenderer.setToolTipText("Tornadic Vortex Signature"); break;}
                    case 8: {headerRenderer.setToolTipText("Mesocyclone Signature"); break;}
                    case 9: {headerRenderer.setToolTipText("Possiblity of severe hail"); break;}
                    case 10: {headerRenderer.setToolTipText("Possibility of hail"); break;}
                    case 11: {headerRenderer.setToolTipText("Predicted max size of hail (in)"); break;}
                    case 12: {headerRenderer.setToolTipText("Vertically integrated liquid water (kg/m^2)"); break;}
                    case 13: {headerRenderer.setToolTipText("Reporting radar"); break;}
                }
                TableColumn column = table.getColumnModel().getColumn(i);
                column.setHeaderRenderer(headerRenderer);
            }
        }*/
        this.getContentPane().add(jsp);
        
        pack();
    }
    
    private java.util.Vector addStorm(Storm storm) {
        java.util.Vector lineData = new java.util.Vector();
        
        String grid = getGrid(storm.getLat(), storm.getLon());
        String id = storm.getID();
        String tvs = storm.getTVS();
        String mso = storm.getMSO();
        String posh = new Double(storm.getPOSH()).toString();
        String poh = new Double(storm.getPOH()).toString();
        String maxSize = new Double(storm.getMXSize()).toString();
        String vil = new Double(storm.getVIL()).toString();
        String dBz =new Integer(storm.getDBz()).toString();
        String ht = new Double(storm.getHT()).toString();
        String top = new Double(storm.getTop()).toString();
        String dir = new Integer(storm.getDirection()).toString();
        if (dir.equals("-1")) {
            dir = "NEW";
        }
        String speed = new Integer(storm.getSpeed()).toString();
        if (speed.equals("-1")) {
            speed = "NEW";
        }
        
        String radar = storm.getRadar();
        
        lineData.add(grid);
        lineData.add(top);
        lineData.add(dBz);
        lineData.add(ht);
        lineData.add(dir);
        lineData.add(speed);
        lineData.add(id);
        lineData.add(tvs);
        lineData.add(mso);
        lineData.add(posh);
        lineData.add(poh);
        lineData.add(maxSize);
        lineData.add(vil);
        lineData.add(radar);
        
        return lineData;
        
        
    }
    private java.util.Vector getColumnHeader() {
        java.util.Vector vect = new java.util.Vector();
        vect.add("Grid");
        vect.add("Top");
        vect.add("dBz");
        vect.add("Height");
        vect.add("Dir");
        vect.add("Speed");
        vect.add("ID");
        vect.add("TVS");
        vect.add("MSO");
        vect.add("POSH");
        vect.add("POH");
        vect.add("MXSize");
        vect.add("VIL");
        vect.add("Rad");
        return vect;
    }
    /**
     * @param args the command line arguments
     */
    
    private String getGrid(double lat, double lon) {
        //field = 20 degrees lon, 10 deg lat
        //grid = 2 degrees lon, 1 deg lat
        //subsquare = 2/24 lon, 1/24 lat;
        //'A' = (char)65
        //'0' (zero)  = (char)(48)
        //West is positive
        String str = "";
        char g1;
        char g2;
        char g3;
        char g4;
        char g5;
        char g6;
        g1 = (char)(Math.floor((-lon + 180)/20.0) + 65);  //x
        g2 = (char)(Math.floor(lat/10.0) + 74);  //x
        
        g3 = (char)(Math.floor(((-lon + 180)%20)/2) + 48); //x
        g4 = (char) (Math.floor((lat + 90)%10) + 48);   //x
        
        g5 = (char) (Math.floor(((-lon + 180)%2)/(2.0/24.0)) + 97);
        g6 = (char) ((Math.floor(((lat + 90)%1)/ (1.0/24.0))) + 97);
        str = str + g1 + g2 + g3+ g4 + g5 + g6;
        
        return str;
        
    }
    /**
     * Returns the JTable that contains the display of the Storms.
     * @return  the JTable of containing the Storms
     */    
    public JTable getTable() {
        return table;
    }
    class MyTableModel extends javax.swing.table.AbstractTableModel implements TableModelListener {
        
        public boolean isCellEditable(int row, int col) {
            return false;
        }
        
        public int getColumnCount() {
            if (stormTableVector.size() > 0) {
                Object obj = stormTableVector.elementAt(0);
                java.util.Vector vect = (java.util.Vector) obj;
                return vect.size();
            }
            else {
                return 0;
            }
        }
        
        public int getRowCount() {
            return stormTableVector.size();
        }
        
        public Object getValueAt(int row, int col) {
            Object obj = stormTableVector.elementAt(row);
            java.util.Vector vect = (java.util.Vector) obj;
            return vect.elementAt(col);
        }
        public String getColumnName(int col) {
            columnNames = getColumnHeader();
            return (String)columnNames.elementAt(col);
        }
        
        public void tableChanged(TableModelEvent e) {
            System.out.println("Table changed!");
        }
        
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    java.util.Vector stormData;
    /**
     * JTable that displays the storm information, one storm per row.
     */    
    public javax.swing.JTable table;
    java.util.Vector columnNames;
    java.util.Vector stormTableVector;
}
